@@ -18,26 +18,29 @@ import math
 import operator as op
 import numpy as np
 import functools
-from typing import Sequence, Tuple, Callable, Union, Optional, cast, List
+from typing import (Sequence, Tuple, Callable, Union, Optional, cast, List, Set,
+                    TYPE_CHECKING)
 
+import jax
 from jax._src import abstract_arrays
 from jax._src import api_util
 from jax._src import basearray
 from jax._src import core
 from jax._src import dispatch
 from jax._src import dtypes
+from jax._src import profiler
 from jax._src.config import config
-from jax._src.util import safe_zip, use_cpp_class, use_cpp_method
+from jax._src.util import use_cpp_class, use_cpp_method
 from jax._src.lib import xla_client as xc
-from jax._src.lib import xla_extension_version
 from jax._src import api
 from jax._src.typing import ArrayLike
 from jax.interpreters import mlir
 from jax._src.interpreters import pxla
 from jax._src.interpreters import xla
-from jax._src.sharding import (
-    Sharding, SingleDeviceSharding, XLACompatibleSharding, PmapSharding,
-    device_replica_id_map)
+from jax._src.sharding import Sharding
+from jax._src.sharding_impls import (
+    SingleDeviceSharding, XLACompatibleSharding, PmapSharding,
+    device_replica_id_map, hashed_index)
 
 Shape = Tuple[int, ...]
 Device = xc.Device
@@ -71,7 +74,7 @@ class Shard:
     except ValueError:
       return f'Shard(device={repr(self.device)}, data={self.data})'
 
-  @property
+  @functools.cached_property
   def index(self) -> Index:
     try:
       device_indices_map_fn = self._sharding.devices_indices_map
@@ -83,7 +86,7 @@ class Shard:
     assert index is not None
     return index
 
-  @property
+  @functools.cached_property
   def replica_id(self) -> int:
     return device_replica_id_map(self._sharding, self._global_shape)[self.device]
 
@@ -105,15 +108,65 @@ def _reconstruct_array(fun, args, arr_state, aval_state):
   return jnp_value
 
 
-def _single_device_array_from_buf(buf, committed):
-  if isinstance(buf, ArrayImpl):
+def _single_device_array_from_buf(buf, committed) -> ArrayImpl:
+  if isinstance(buf, ArrayImpl) and buf._committed == committed:  # type: ignore
     return buf
   db = dispatch._set_aval(buf)
   return ArrayImpl(db.aval, SingleDeviceSharding(db.device()), [db],
                    committed=committed, _skip_checks=True)
 
 
-@use_cpp_class(xc.ArrayImpl)
+def _is_reduced_on_dim(idx):
+  # TODO(yashkatariya): This handles very narrow use case where we know XLA will
+  # not return an output with uneven sharding. Remove this after we have the
+  # ability to catch uneven shardings in lower_sharding_computation and raise
+  # a special exception for that which can be caught here to fallback to
+  # bouncing via host.
+  if not isinstance(idx, tuple):
+    idx = (idx,)
+  return all(isinstance(i, int) or
+             (isinstance(i, slice) and  i == slice(None)) or
+             (isinstance(i, (np.ndarray, jax.Array)) and not i.shape and
+              np.issubdtype(i.dtype, np.integer))
+             for i in idx)
+
+
+@functools.lru_cache(maxsize=4096)
+def _cached_index_calc(s, shape):
+  map_ = s.addressable_devices_indices_map(shape)
+  seen_h_indices = set()
+  m = {}
+  for d, index in map_.items():
+    h_index = hashed_index(index)
+    if h_index not in seen_h_indices:
+      seen_h_indices.add(h_index)
+      m[d] = index
+  return m
+
+
+def _create_copy_plan(arrays, s: Sharding, shape: Shape):
+  di_map = _cached_index_calc(s, shape)
+  copy_plan = []
+  for a in arrays:
+    ind = di_map.get(a.device(), None)
+    if ind is not None:
+      copy_plan.append((ind, a))
+  return copy_plan
+
+
+@functools.lru_cache(maxsize=4096)
+def _process_has_full_value_in_mcjax(s, shape):
+  # Return False for single host as a fast path.
+  if jax.process_count() == 1:
+    return False
+
+  num_unique_indices = len(
+      set(hashed_index(v) for v in s.devices_indices_map(shape).values()))
+  num_addressable_unique_indices = len(
+      set(hashed_index(v) for v in s.addressable_devices_indices_map(shape).values()))
+  return num_unique_indices == num_addressable_unique_indices
+
+
 class ArrayImpl(basearray.Array):
   # TODO(yashkatariya): Add __slots__ here.
 
@@ -271,11 +324,7 @@ class ArrayImpl(basearray.Array):
     from jax._src.numpy import lax_numpy
     self._check_if_deleted()
 
-    if dispatch.is_single_device_sharding(self.sharding) or self.is_fully_replicated:
-      return lax_numpy._rewriting_take(self, idx)
-    # TODO(yashkatariya): Make it work for other Shardings too wherever its
-    # possible to not do data movement.
-    elif isinstance(self.sharding, PmapSharding):
+    if isinstance(self.sharding, PmapSharding):
       if not isinstance(idx, tuple):
         cidx = (idx,) + (slice(None),) * (len(self.shape) - 1)
       else:
@@ -283,20 +332,15 @@ class ArrayImpl(basearray.Array):
       if self._npy_value is None:
         indices = tuple(self.sharding.devices_indices_map(self.shape).values())
         try:
-          buf_idx = indices.index(cidx)
+          arr_idx = indices.index(cidx)
         except ValueError:
-          buf_idx = None
-        if buf_idx is not None:
-          buf = self._arrays[buf_idx]
-          aval = core.ShapedArray(buf.shape, self.dtype)
-          return ArrayImpl(aval, SingleDeviceSharding(buf.device()), [buf],
-                           committed=False, _skip_checks=True)
+          arr_idx = None
+        if arr_idx is not None:
+          arr = self._arrays[arr_idx]
+          return _single_device_array_from_buf(arr, committed=False)
       return lax_numpy._rewriting_take(self, idx)
     else:
-      # TODO(yashkatariya): Don't bounce to host and use `_rewriting_take` or
-      # the fast path (see PmapSharding branch above) after after uneven
-      # partitioning support is added
-      return api.device_put(self._value[idx])
+      return lax_numpy._rewriting_take(self, idx)
 
   def __iter__(self):
     if self.ndim == 0:
@@ -352,6 +396,7 @@ class ArrayImpl(basearray.Array):
                   'named_shape': self.aval.named_shape}
     return (_reconstruct_array, (fun, args, arr_state, aval_state))
 
+  @use_cpp_method()
   def unsafe_buffer_pointer(self):
     if len(self._arrays) != 1:
       raise ValueError("unsafe_buffer_pointer() is supported only for unsharded"
@@ -359,19 +404,18 @@ class ArrayImpl(basearray.Array):
     return self._arrays[0].unsafe_buffer_pointer()
 
   @property
+  @use_cpp_method()
   def __cuda_array_interface__(self):
     if len(self._arrays) != 1:
       raise ValueError("__cuda_array_interface__() is supported only for "
                        "unsharded arrays.")
     return self._arrays[0].__cuda_array_interface__  # pytype: disable=attribute-error  # bind-properties
 
+  @use_cpp_method()
   def on_device_size_in_bytes(self):
     """Returns the total global on-device size of the array in bytes."""
     arr = self._arrays[0]
-    if hasattr(arr, "_on_device_size_in_bytes"):
-      per_shard_size = arr._on_device_size_in_bytes()  # type: ignore
-    else:
-      per_shard_size = arr.on_device_size_in_bytes()  # type: ignore
+    per_shard_size = arr.on_device_size_in_bytes()  # type: ignore
     return per_shard_size * len(self.sharding.device_set)
 
   # TODO(yashkatariya): Remove this method when everyone is using devices().
@@ -384,9 +428,9 @@ class ArrayImpl(basearray.Array):
     raise ValueError('Length of devices is greater than 1. '
                      'Please use `.devices()`.')
 
-  def devices(self) -> List[Device]:
+  def devices(self) -> Set[Device]:
     self._check_if_deleted()
-    return list(self.sharding.device_set)
+    return self.sharding.device_set
 
   # TODO(https://github.com/google/jax/issues/12380): Remove this when DA is
   # deleted.
@@ -443,6 +487,7 @@ class ArrayImpl(basearray.Array):
       out.append(Shard(global_d, self.sharding, self.shape, array))
     return out
 
+  @use_cpp_method()
   def delete(self):
     if self._arrays is None:
       return
@@ -471,55 +516,65 @@ class ArrayImpl(basearray.Array):
       db.block_until_ready()
     return self
 
+  @use_cpp_method()
+  def _single_device_array_to_np_array(self):
+    return np.asarray(self._arrays[0])
+
+  @use_cpp_method()
+  def _copy_single_device_array_to_host_async(self):
+    self._arrays[0].copy_to_host_async()
+
+  @profiler.annotate_function
   def copy_to_host_async(self):
     self._check_if_deleted()
     if self._npy_value is None:
       if self.is_fully_replicated:
-        self._arrays[0].copy_to_host_async()
+        self._copy_single_device_array_to_host_async()
         return
-      try:
-        self.addressable_shards[0].replica_id
-        replica_id_exists = True
-      except ValueError:
-        replica_id_exists = False
-
-      for s in self.addressable_shards:
-        if not replica_id_exists or s.replica_id == 0:
-          s.data._arrays[0].copy_to_host_async()  # pytype: disable=attribute-error
+      copy_plan = _create_copy_plan(self._arrays, self.sharding, self.shape)
+      for _, arr in copy_plan:
+        arr._copy_single_device_array_to_host_async()
 
   @property
+  @functools.partial(profiler.annotate_function, name="np.asarray(jax.Array)")
   def _value(self) -> np.ndarray:
     self._check_if_deleted()
 
     if self._npy_value is None:
       if self.is_fully_replicated:
-        self._npy_value = np.asarray(self._arrays[0])  # type: ignore
+        self._npy_value = self._single_device_array_to_np_array()  # type: ignore
         self._npy_value.flags.writeable = False
         return cast(np.ndarray, self._npy_value)
 
-      if not self.is_fully_addressable:
+      # TODO(yashkatariya): Merge `_process_has_full_value_in_mcjax` with
+      # is_fully_addressable.
+      if (not self.is_fully_addressable and
+          not _process_has_full_value_in_mcjax(self.sharding, self.shape)):
         raise RuntimeError("Fetching value for `jax.Array` that spans "
                            "non-addressable devices is not possible. You can use "
                            "`jax.experimental.multihost_utils.process_allgather` "
                            "for this use case.")
 
-      self.copy_to_host_async()
+      copy_plan = _create_copy_plan(self._arrays, self.sharding, self.shape)
+      for _, arr in copy_plan:
+        arr._copy_single_device_array_to_host_async()
+
       npy_value = np.empty(self.shape, self.dtype)
-
-      try:
-        self.addressable_shards[0].replica_id
-        replica_id_exists = True
-      except ValueError:
-        replica_id_exists = False
-
-      for s in self.addressable_shards:
-        if not replica_id_exists or s.replica_id == 0:
-          npy_value[s.index] = np.asarray(s.data._arrays[0])  # type: ignore  # [union-attr]
+      for ind, arr in copy_plan:
+        npy_value[ind] = arr._single_device_array_to_np_array()
       self._npy_value = npy_value  # type: ignore
       self._npy_value.flags.writeable = False
     # https://docs.python.org/3/library/typing.html#typing.cast
     return cast(np.ndarray, self._npy_value)
 
+
+# TODO(b/273265390): ideally we would write this as a decorator on the ArrayImpl
+# class, however this triggers a pytype bug. Workaround: apply the decorator
+# after the fact.
+if not TYPE_CHECKING:
+  ArrayImpl = use_cpp_class(xc.ArrayImpl)(ArrayImpl)
+
+
 # explicitly set to be unhashable. Same as what device_array.py does.
 setattr(ArrayImpl, "__hash__", None)
 setattr(ArrayImpl, "__array_priority__", 100)
@@ -576,7 +631,8 @@ def make_array_from_callback(
 
 
 def make_array_from_single_device_arrays(
-    shape: Shape, sharding: Sharding, arrays: Sequence[ArrayImpl]) -> ArrayImpl:
+    shape: Shape, sharding: Sharding, arrays: Sequence[basearray.Array]
+) -> ArrayImpl:
   r"""Returns a ``jax.Array`` from a sequence of ``jax.Array``\s on a single device.
 
   ``jax.Array`` on a single device is analogous to a ``DeviceArray``. You can use
@@ -616,7 +672,10 @@ def make_array_from_single_device_arrays(
   # All input arrays should be committed. Checking it is expensive on
   # single-controller systems.
   aval = core.ShapedArray(shape, arrays[0].dtype, weak_type=False)
-  return ArrayImpl(aval, sharding, arrays, committed=True)
+  # TODO(phawkins): ideally the cast() could be checked. Revisit this after
+  # removing DeviceArray.
+  return ArrayImpl(aval, sharding, cast(Sequence[ArrayImpl], arrays),
+                   committed=True)
 
 
 core.pytype_aval_mappings[ArrayImpl] = abstract_arrays.canonical_concrete_aval
@@ -639,21 +698,14 @@ def _array_shard_arg(x, devices, indices, sharding):
   x_indices = x.sharding.addressable_devices_indices_map(x.shape).values()
   if not x.is_fully_addressable:
     if tuple(x_indices) == tuple(indices):
-      if xla_extension_version >= 136:
-        return x
-      else:
-        return x._arrays
+      return x
     else:
       raise NotImplementedError(
           "Cannot reshard an input that is not fully addressable")
   else:
     if tuple(x_indices) == tuple(indices):
-      if xla_extension_version >= 136:
-        return xc.copy_array_to_devices_with_sharding(
-            x, list(devices), sharding)
-      else:
-        return [buf if buf.device() == d else buf.copy_to_device(d)
-                for buf, d in safe_zip(x._arrays, devices)]
+      return xc.copy_array_to_devices_with_sharding(
+          x, list(devices), sharding)
     # Resharding starts here:
     if dispatch.is_single_device_sharding(x.sharding):
       return pxla.shard_device_array(x, devices, indices, sharding)
@@ -672,15 +724,12 @@ def _array_global_result_handler(global_aval, out_sharding, committed,
   if core.is_opaque_dtype(global_aval.dtype):
     return global_aval.dtype._rules.global_sharded_result_handler(
         global_aval, out_sharding, committed, is_out_sharding_from_xla)
-  if xla_extension_version >= 131:
-    return xc.array_result_handler(
-        global_aval, out_sharding, committed=committed, _skip_checks=True
-    )
-  return lambda bufs: ArrayImpl(global_aval, out_sharding, bufs,
-                                committed=committed, _skip_checks=True)
-pxla.global_result_handlers[(core.ShapedArray, pxla.OutputType.Array)] = _array_global_result_handler
-pxla.global_result_handlers[(core.ConcreteArray, pxla.OutputType.Array)] = _array_global_result_handler
-pxla.global_result_handlers[(core.AbstractToken, pxla.OutputType.Array)] = lambda *_: lambda *_: core.token
+  return xc.array_result_handler(
+      global_aval, out_sharding, committed=committed, _skip_checks=True
+  )
+pxla.global_result_handlers[core.ShapedArray] = _array_global_result_handler
+pxla.global_result_handlers[core.ConcreteArray] = _array_global_result_handler
+pxla.global_result_handlers[core.AbstractToken] = lambda *_: lambda *_: core.token
 
 
 # Only used for Arrays that come out of pmap.
@@ -690,11 +739,8 @@ def _array_local_result_handler(aval, sharding, indices):
   if core.is_opaque_dtype(aval.dtype):
     return aval.dtype._rules.local_sharded_result_handler(
         aval, sharding, indices)
-  if xla_extension_version >= 131:
-    return xc.array_result_handler(
-        aval, sharding, committed=True, _skip_checks=True
-    )
-  return lambda bufs: ArrayImpl(aval, sharding, bufs, committed=True,
-                                _skip_checks=True)
-pxla.local_result_handlers[(core.ShapedArray, pxla.OutputType.Array)] = _array_local_result_handler
-pxla.local_result_handlers[(core.ConcreteArray, pxla.OutputType.Array)] = _array_local_result_handler
+  return xc.array_result_handler(
+      aval, sharding, committed=True, _skip_checks=True
+  )
+pxla.local_result_handlers[core.ShapedArray] = _array_local_result_handler
+pxla.local_result_handlers[core.ConcreteArray] = _array_local_result_handler
