@@ -19,7 +19,6 @@ import logging
 import os
 import sys
 import threading
-import warnings
 from typing import Any, List, Callable, Hashable, NamedTuple, Iterator, Optional
 
 from jax._src import lib
@@ -675,22 +674,13 @@ jax2tf_associative_scan_reductions = config.define_bool_state(
     )
 )
 
-jax2tf_default_experimental_native_lowering = config.define_bool_state(
-    name='jax2tf_default_experimental_native_lowering',
-    default=bool_env('JAX2TF_DEFAULT_EXPERIMENTAL_NATIVE_LOWERING', False),
+jax2tf_default_native_serialization = config.define_bool_state(
+    name='jax2tf_default_native_serialization',
+    default=bool_env('JAX2TF_DEFAULT_NATIVE_SERIALIZATION', False),
     help=(
-        'DO NOT USE, highly experimental. Sets the default value of the '
-        'experimental_native_lowering parameter to jax2tf.convert.'
-    )
-)
-
-jax2tf_use_stablehlo = config.define_bool_state(
-    name='jax2tf_use_stablehlo',
-    default=bool_env('JAX2TF_USE_STABLEHLO', True),
-    help=(
-        'DO NOT USE, highly experimental. Use in conjunction with jax2tf '
-        'experimental_native_lowering, to use StableHLO instead of MHLO as '
-        'the serialization format.'
+        'Sets the default value of the native_serialization parameter to '
+        'jax2tf.convert. Prefer using the parameter instead of the flag, '
+        'the flag may be removed in the future.'
     )
 )
 
@@ -749,6 +739,13 @@ log_compiles = config.define_bool_state(
           'option is set, the log level is WARNING; otherwise the level is '
           'DEBUG.'))
 
+log_compiles = config.define_bool_state(
+    name='jax_log_checkpoint_residuals',
+    default=False,
+    help=('Log a message every time jax.checkpoint (aka jax.remat) is '
+          'partially evaluated (e.g. for autodiff), printing what residuals '
+          'are saved.'))
+
 parallel_functions_output_gda = config.define_bool_state(
     name='jax_parallel_functions_output_gda',
     default=False,
@@ -756,21 +753,17 @@ parallel_functions_output_gda = config.define_bool_state(
 
 def _update_jax_array_global(val):
   if val is not None and not val:
-    warnings.warn(
-        'DeviceArray, ShardedDeviceArray, and GlobalDeviceArray have been '
-        'deprecated. Please use `jax.Array`. See '
-        'https://jax.readthedocs.io/en/latest/jax_array_migration.html on how '
-        'to migrate to `jax.Array`.', DeprecationWarning)
-  lib.jax_jit.global_state().jax_array = val
+    raise ValueError(
+        'jax.config.jax_array cannot be disabled after jax 0.4.7 release.'
+        ' Please downgrade to jax and jaxlib 0.4.6 if you want to disable'
+        ' jax.config.jax_array.')
 
 def _update_jax_array_thread_local(val):
   if val is not None and not val:
-    warnings.warn(
-        'DeviceArray, ShardedDeviceArray, and GlobalDeviceArray have been '
-        'deprecated. Please use `jax.Array`. See '
-        'https://jax.readthedocs.io/en/latest/jax_array_migration.html on how '
-        'to migrate to `jax.Array`.', DeprecationWarning)
-  lib.jax_jit.thread_local_state().jax_array = val
+    raise ValueError(
+        'jax.config.jax_array cannot be disabled after jax 0.4.7 release.'
+        ' Please downgrade to jax and jaxlib 0.4.6 if you want to disable'
+        ' jax.config.jax_array.')
 
 jax_array = config.define_bool_state(
     name='jax_array',
