@@ -27,18 +27,18 @@ from jax import lax
 from jax._src import core
 from jax._src import effects
 from jax._src import linear_util as lu
+from jax._src import mesh as mesh_lib
 from jax._src import pjit
 from jax._src import util
 from jax._src.interpreters import ad
 from jax._src.interpreters import batching
 from jax._src.interpreters import mlir
-from jax._src.interpreters import pxla
 from jax._src.lib import xla_client as xc
 from jax._src.lib.mlir import ir
 from jax._src.lib.mlir.dialects import hlo
-from jax._src.sharding import Sharding, GSPMDSharding, NamedSharding
-from jax.interpreters import partial_eval as pe
-from jax.config import config
+from jax._src.sharding import Sharding
+from jax._src.sharding_impls import GSPMDSharding, NamedSharding
+from jax._src.interpreters import partial_eval as pe
 
 # pytype: disable=import-error
 try:
@@ -276,8 +276,6 @@ inspect_sharding_p = core.Primitive("inspect_sharding")
 inspect_sharding_p.multiple_results = True
 
 def _inspect_sharding_impl(value, *, callback):
-  if not config.jax_array:
-    raise NotImplementedError("`inspect_sharding` not implemented.")
   callback(value.sharding)
   return []
 inspect_sharding_p.def_impl(_inspect_sharding_impl)
@@ -296,7 +294,7 @@ batching.primitive_batchers[inspect_sharding_p] = (
     _inspect_sharding_batching_rule)
 
 def _inspect_sharding_jvp_rule(primals, _, **params):
-  return inspect_sharding_p.bind(*primals, **params)
+  return inspect_sharding_p.bind(*primals, **params), []
 ad.primitive_jvps[inspect_sharding_p] = _inspect_sharding_jvp_rule
 
 sharding_callbacks = weakref.WeakValueDictionary()  # type: ignore
@@ -310,7 +308,7 @@ class ShardingCallbackInfo:
 def _inspect_sharding_lowering_rule(ctx: mlir.LoweringRuleContext, value, *,
                                     callback):
 
-  mesh = pxla.thread_resources.env.physical_mesh
+  mesh = mesh_lib.thread_resources.env.physical_mesh
   axis_context = ctx.module_context.axis_context
 
   if isinstance(axis_context, mlir.ShardingContext):
@@ -598,8 +596,6 @@ def inspect_array_sharding(value, *, callback: Callable[[Sharding], None]):
 
 def visualize_array_sharding(arr, **kwargs):
   """Visualizes an array's sharding."""
-  if not config.jax_array:
-    raise NotImplementedError("`visualize_array_sharding` not implemented.")
   def _visualize(sharding):
     return visualize_sharding(arr.shape, sharding, **kwargs)
   inspect_array_sharding(arr, callback=_visualize)
