@@ -15,6 +15,7 @@
 # On-device arrays.
 
 from functools import partial, partialmethod
+import math
 import operator
 from typing import (Any, List, Optional, Union)
 import weakref
@@ -26,7 +27,6 @@ import jax
 from jax._src import core
 from jax._src import abstract_arrays
 from jax._src import profiler
-from jax._src import util
 from jax._src.config import config
 from jax._src.lib import xla_client as xc
 from jax._src.typing import Array
@@ -59,6 +59,7 @@ def make_device_array(
   This is to be used only within JAX. It will return either a PythonDeviceArray
   or a C++ equivalent implementation.
   """
+
   if isinstance(device_buffer, xc.Buffer):
 
     if device_buffer.aval == aval and device_buffer._device == device:
@@ -125,6 +126,7 @@ class _DeviceArray(DeviceArray):  # type: ignore
           aval, npy_value.shape, npy_value.dtype)
       assert (device is None) or device is device_buffer.device()
 
+
   def _check_if_deleted(self):
     if self.device_buffer is deleted_buffer:
       raise RuntimeError("DeviceArray has been deleted.")
@@ -161,7 +163,7 @@ class _DeviceArray(DeviceArray):  # type: ignore
 
   @property
   def size(self):
-    return util.prod(self.aval.shape)
+    return math.prod(self.aval.shape)
 
   @property
   def ndim(self):
