@@ -18,7 +18,7 @@ import os
 
 from typing import Any, Optional, Union, Sequence
 
-from jax._src.clusters import ClusterEnv
+from jax._src import clusters
 from jax._src.config import config
 from jax._src.lib import xla_extension
 
@@ -30,6 +30,7 @@ class State:
   service: Optional[Any] = None
   client: Optional[Any] = None
   preemption_sync_manager: Optional[Any] = None
+  coordinator_address: Optional[str] = None
 
   def initialize(self,
                  coordinator_address: Optional[str] = None,
@@ -41,11 +42,11 @@ class State:
     if isinstance(local_device_ids, int):
       local_device_ids = [local_device_ids]
 
-    (coordinator_address,
-     num_processes,
-     process_id,
-     local_device_ids) = ClusterEnv.auto_detect_unset_distributed_params(
-      coordinator_address, num_processes, process_id, local_device_ids)
+    (coordinator_address, num_processes, process_id, local_device_ids) = (
+        clusters.ClusterEnv.auto_detect_unset_distributed_params(
+            coordinator_address, num_processes, process_id, local_device_ids
+        )
+    )
 
     if coordinator_address is None:
       raise ValueError('coordinator_address should be defined.')
@@ -54,6 +55,8 @@ class State:
     if process_id is None:
       raise ValueError('The process id of the current process must be defined.')
 
+    self.coordinator_address = coordinator_address
+
     if local_device_ids:
       visible_devices = ','.join(str(x) for x in local_device_ids) # type: ignore[union-attr]
       logger.info('JAX distributed initialized with visible devices: %s', visible_devices)
