@@ -13,13 +13,12 @@
 # limitations under the License.
 
 from jax import numpy as jnp
-from jax._src import core
 from jax._src import device_array
-from jax._src import dispatch
 from jax._src import array
 from jax._src import xla_bridge
 from jax._src.lib import xla_client
 
+
 SUPPORTED_DTYPES = frozenset({
     jnp.int8, jnp.int16, jnp.int32, jnp.int64, jnp.uint8, jnp.uint16,
     jnp.uint32, jnp.uint64, jnp.float16, jnp.bfloat16, jnp.float32,
@@ -40,16 +39,12 @@ def to_dlpack(x: device_array.DeviceArrayProtocol, take_ownership: bool = False)
       undefined behavior if the DLPack consumer writes to a buffer that JAX
       owns.
   """
-  if not isinstance(x, (device_array.DeviceArray, array.ArrayImpl)):
-    raise TypeError("Argument to to_dlpack must be a DeviceArray or Array, got {}"
-                    .format(type(x)))
-  if isinstance(x, array.ArrayImpl):
-    assert len(x._arrays) == 1
-    buf = x._arrays[0]
-  else:
-    buf = x.device_buffer
+  if not isinstance(x, array.ArrayImpl):
+    raise TypeError("Argument to to_dlpack must be a jax.Array, "
+                    f"got {type(x)}")
+  assert len(x.devices()) == 1
   return xla_client._xla.buffer_to_dlpack_managed_tensor(
-      buf, take_ownership=take_ownership)
+      x.addressable_data(0), take_ownership=take_ownership)  # type: ignore
 
 def from_dlpack(dlpack):
   """Returns a ``DeviceArray`` representation of a DLPack tensor.
@@ -72,13 +67,5 @@ def from_dlpack(dlpack):
     except RuntimeError:
       gpu_backend = None
 
-  buf = xla_client._xla.dlpack_managed_tensor_to_buffer(
-      dlpack, cpu_backend, gpu_backend)
-  if isinstance(buf, array.ArrayImpl):
-    aval = buf.aval
-  else:
-    xla_shape = buf.xla_shape()
-    assert not xla_shape.is_tuple()
-    aval = core.ShapedArray(xla_shape.dimensions(), xla_shape.numpy_dtype())
-  return jnp.asarray(           # asarray ensures dtype canonicalization
-      dispatch.maybe_create_array_from_da(buf, aval, buf.device()))
+  return jnp.asarray(xla_client._xla.dlpack_managed_tensor_to_buffer(
+      dlpack, cpu_backend, gpu_backend))
