@@ -35,7 +35,7 @@ from jax._src import linear_util as lu
 from jax._src.util import (unzip2, unzip3, safe_map, safe_zip, split_list,
                            canonicalize_axis, moveaxis, as_hashable_function,
                            curry, memoize, weakref_lru_cache)
-from jax.interpreters import partial_eval as pe
+from jax._src.interpreters import partial_eval as pe
 
 Array = Any
 map, unsafe_map = safe_map, map
@@ -815,7 +815,7 @@ def _matchaxis_symbolic_zeros(axis_name, sz, name, src, dst, x, sum_match=False)
 
 ### utilities for defining primitives' batching rules
 
-BatchingRule = Callable[..., Tuple[Any, Union[int, Tuple[int, ...]]]]
+BatchingRule = Callable[..., Tuple[Any, Union[None, int, Tuple[Union[None, int], ...]]]]
 primitive_batchers : Dict[core.Primitive, BatchingRule] = {}
 axis_primitive_batchers: Dict[core.Primitive, Callable] = {}
 spmd_axis_primitive_batchers: Dict[core.Primitive, Callable] = {}
