@@ -32,9 +32,6 @@ import numpy as np
 
 from jax._src import linear_util as lu
 from jax.config import config
-from jax._src.interpreters import ad
-from jax.interpreters import partial_eval as pe
-from jax._src.interpreters import xla
 from jax._src import ad_util
 from jax._src import core
 from jax._src import device_array
@@ -43,6 +40,9 @@ from jax._src import effects as effects_lib
 from jax._src import source_info_util
 from jax._src import util
 from jax._src import xla_bridge as xb
+from jax._src.interpreters import ad
+from jax._src.interpreters import partial_eval as pe
+from jax._src.interpreters import xla
 from jax._src.lib import xla_client as xc
 from jax._src.lib.mlir import ir
 from jax._src.lib.mlir.dialects import hlo
@@ -299,7 +299,7 @@ for _scalar_type in [np.int8, np.int16, np.int32, np.int64,
                      np.float16, np.float32, np.float64,
                      np.complex64, np.complex128,
                      np.bool_, np.longlong, dtypes.bfloat16]:
-  register_constant_handler(_scalar_type, _ndarray_constant_handler)
+  register_constant_handler(_scalar_type, _ndarray_constant_handler)  # type: ignore
 
 def _python_scalar_handler(dtype, val, canonicalize_dtypes):
   return _numpy_array_constant(np.array(val, dtype), canonicalize_dtypes)
@@ -667,8 +667,8 @@ def lower_jaxpr_to_module(
     replicated_args: Optional[Sequence[bool]] = None,
     arg_shardings: Optional[Sequence[Optional[xc.OpSharding]]] = None,
     result_shardings: Optional[Sequence[Optional[xc.OpSharding]]] = None,
-    arg_names: Optional[Sequence[str]] = None,
-    result_names: Optional[Sequence[str]] = None,
+    arg_names: Optional[Sequence[Optional[str]]] = None,
+    result_names: Optional[Sequence[Optional[str]]] = None,
 ) -> LoweringResult:
   """Lowers a top-level jaxpr to an MLIR module.
 
@@ -863,8 +863,8 @@ def lower_jaxpr_to_fun(
     input_output_aliases: Optional[Sequence[Optional[int]]] = None,
     num_output_tokens: int = 0,
     api_name: str = 'jit',
-    arg_names: Optional[Sequence[str]] = None,
-    result_names: Optional[Sequence[str]] = None,
+    arg_names: Optional[Sequence[Optional[str]]] = None,
+    result_names: Optional[Sequence[Optional[str]]] = None,
 ) -> func_dialect.FuncOp:
   """Lowers jaxpr and its callees to an IR function.
 
@@ -985,10 +985,10 @@ def lower_jaxpr_to_fun(
         if alias is not None:
           attrs["tf.aliasing_output"] = i32_attr(alias)
 
-    if config.jax_jit_pjit_api_merge and arg_names:
+    if arg_names:
       named_arg_attrs = arg_attrs[num_dim_vars + num_tokens:]
-      if len(named_arg_attrs) == len(arg_names):
-        for attrs, name_ in zip(named_arg_attrs, arg_names):
+      for attrs, name_ in zip(named_arg_attrs, arg_names):
+        if name_:
           attrs['jax.arg_info'] = ir.StringAttr.get(name_)
 
     func_op.arg_attrs = ir.ArrayAttr.get(
@@ -997,7 +997,7 @@ def lower_jaxpr_to_fun(
   result_attrs: List[Dict[str, ir.Attribute]] = [
       {} for _ in range(len(flat_output_types))]
 
-  if config.jax_jit_pjit_api_merge and result_names:
+  if result_names:
     named_result_attrs = result_attrs[num_tokens:]
     if len(named_result_attrs) == len(result_names):
       for attrs, name_ in zip(named_result_attrs, result_names):
@@ -1008,13 +1008,6 @@ def lower_jaxpr_to_fun(
       if sharding is not None:
         attrs['mhlo.sharding'] = get_sharding_attr(sharding)
 
-    # func_op.result_attrs = ir.ArrayAttr.get([
-    #         ir.DictAttr.get(
-    #         {} if sharding is None else
-    #         {"mhlo.sharding": get_sharding_attr(sharding)}
-    #     ) for sharding in ir_result_shardings
-    # ])
-
   func_op.result_attrs = ir.ArrayAttr.get(
       [ir.DictAttr.get(attrs) for attrs in result_attrs])
 
@@ -1283,20 +1276,7 @@ def _call_lowering(fn_name, stack_name, call_jaxpr, backend, ctx, avals_in,
   tokens_out = tokens_in.update_tokens(TokenSet(zip(effects, tokens)))
   return out_nodes, tokens_out
 
-def _xla_call_lower(ctx, *args,
-                    backend=None, name, call_jaxpr, donated_invars, inline=None,
-                    device=None, keep_unused=None):
-  del device, donated_invars, inline, keep_unused  # Ignored.
-  out_nodes, tokens = _call_lowering(
-      name, util.wrap_name(name, "jit"), call_jaxpr, backend,
-      ctx.module_context, ctx.avals_in, ctx.avals_out, ctx.tokens_in,
-      *args, dim_var_values=ctx.dim_var_values)
-  ctx.set_tokens_out(tokens)
-  return out_nodes
-
-register_lowering(xla.xla_call_p, _xla_call_lower)
-
-def _core_call_lowering(ctx, *args, name, backend=None, call_jaxpr):
+def core_call_lowering(ctx, *args, name, backend=None, call_jaxpr):
   out_nodes, tokens = _call_lowering(
       name, name, call_jaxpr, backend, ctx.module_context,
       ctx.avals_in, ctx.avals_out, ctx.tokens_in, *args,
@@ -1304,9 +1284,9 @@ def _core_call_lowering(ctx, *args, name, backend=None, call_jaxpr):
   ctx.set_tokens_out(tokens)
   return out_nodes
 
-register_lowering(core.call_p, partial(_core_call_lowering, name="core_call"))
+register_lowering(core.call_p, partial(core_call_lowering, name="core_call"))
 register_lowering(core.closed_call_p,
-                  partial(_core_call_lowering, name="core_closed_call"))
+                  partial(core_call_lowering, name="core_closed_call"))
 
 def broadcast_in_dim(ctx: LoweringRuleContext, op, aval_out: core.AbstractValue, *,
                      broadcast_dimensions) -> ir.Value:
@@ -1533,10 +1513,7 @@ def set_sharding(op, sharding_proto: xc.OpSharding):
 
 
 def get_sharding_attr(sharding_proto: xc.OpSharding):
-  if xc.mlir_api_version >= 44:
-    return ir.StringAttr.get(repr(xc.HloSharding.from_proto(sharding_proto)))
-  else:
-    return ir.StringAttr.get(sharding_proto.SerializeToString())
+  return ir.StringAttr.get(repr(xc.HloSharding.from_proto(sharding_proto)))
 
 
 # MLIR lowerings for lax primitives
