@@ -23,10 +23,10 @@ import numpy as np
 
 import jax
 from jax import lax
-from jax.interpreters import xla
 
 from jax._src import ad_util
 from jax._src import api
+from jax._src import dispatch
 from jax._src import dtypes
 from jax._src.core import (
     Primitive, ShapedArray, raise_to_shaped, is_constant_shape)
@@ -448,10 +448,12 @@ mlir.register_lowering(
 # Asymmetric eigendecomposition
 
 def eig_impl(operand, *, compute_left_eigenvectors, compute_right_eigenvectors):
-  return (
-    xla.apply_primitive(eig_p, operand,
-                        compute_left_eigenvectors=compute_left_eigenvectors,
-                        compute_right_eigenvectors=compute_right_eigenvectors))
+  return dispatch.apply_primitive(
+      eig_p,
+      operand,
+      compute_left_eigenvectors=compute_left_eigenvectors,
+      compute_right_eigenvectors=compute_right_eigenvectors,
+  )
 
 def eig_lower(*args, **kw):
   raise NotImplementedError(
@@ -577,8 +579,8 @@ def eigh_jacobi(x: ArrayLike, *, lower: bool = True,
   return w, v
 
 def _eigh_jacobi_impl(operand, *, lower, sort_eigenvalues):
-  w, v = xla.apply_primitive(eigh_jacobi_p, operand, lower=lower,
-                             sort_eigenvalues=sort_eigenvalues)
+  w, v = dispatch.apply_primitive(eigh_jacobi_p, operand, lower=lower,
+                                  sort_eigenvalues=sort_eigenvalues)
   return w, v
 
 def _eigh_jacobi_abstract_eval(operand, *, lower, sort_eigenvalues):
@@ -597,24 +599,45 @@ def _eigh_jacobi_abstract_eval(operand, *, lower, sort_eigenvalues):
     w, v = operand, operand
   return w, v
 
-def _eigh_jacobi_translation_rule(ctx, avals_in, avals_out, operand, *, lower,
-                                  sort_eigenvalues):
-  operand_aval, = avals_in
+
+def _eigh_jacobi_lowering_rule(ctx, operand, lower, sort_eigenvalues):
+  operand_aval, = ctx.avals_in
   if operand_aval.shape[-1] == 0:
-    return [xops.Real(xops.Reshape(operand, operand_aval.shape[:-1])), operand]
-  v, w = xops.Eigh(operand, lower=lower, sort_eigenvalues=sort_eigenvalues)
-  return w, v
+    reshape_aval = operand_aval.update(shape=operand_aval.shape[:-1])
+    return [
+        hlo.RealOp(mlir.reshape(ctx, operand, reshape_aval)).result,
+        operand,
+    ]
+
+  eigvals_type = mlir.aval_to_ir_type(ctx.avals_out[0])
+  eigvecs_type = mlir.aval_to_ir_type(ctx.avals_out[1])
+  eigh_type = ir.TupleType.get_tuple([eigvecs_type, eigvals_type])
+
+  backend_config = f"{int(lower)},{int(sort_eigenvalues)},100,1e-6"
+  op = hlo.CustomCallOp(
+      [eigh_type],
+      [operand],
+      call_target_name=ir.StringAttr.get("Eigh"),
+      has_side_effect=ir.BoolAttr.get(False),
+      backend_config=ir.StringAttr.get(backend_config),
+      api_version=mlir.i32_attr(1),
+  )
+  return (
+      hlo.GetTupleElementOp(op, 1).result,
+      hlo.GetTupleElementOp(op, 0).result,
+  )
+
 
 eigh_jacobi_p = Primitive('eigh_jacobi')
 eigh_jacobi_p.multiple_results = True
 eigh_jacobi_p.def_impl(_eigh_jacobi_impl)
 eigh_jacobi_p.def_abstract_eval(_eigh_jacobi_abstract_eval)
-xla.register_translation(eigh_jacobi_p, _eigh_jacobi_translation_rule)
+mlir.register_lowering(eigh_jacobi_p, _eigh_jacobi_lowering_rule)
 
 
 def _eigh_impl(operand, *, lower, sort_eigenvalues):
-  v, w = xla.apply_primitive(eigh_p, operand, lower=lower,
-                             sort_eigenvalues=sort_eigenvalues)
+  v, w = dispatch.apply_primitive(eigh_p, operand, lower=lower,
+                                  sort_eigenvalues=sort_eigenvalues)
   return v, w
 
 def _eigh_abstract_eval(operand, *, lower, sort_eigenvalues):
@@ -995,7 +1018,7 @@ def _lu_pivots_to_permutation_gpu_lowering(lowering, ctx, pivots, *,
 lu_pivots_to_permutation_p = Primitive('lu_pivots_to_permutation')
 lu_pivots_to_permutation_p.multiple_results = False
 lu_pivots_to_permutation_p.def_impl(
-    partial(xla.apply_primitive, lu_pivots_to_permutation_p))
+    partial(dispatch.apply_primitive, lu_pivots_to_permutation_p))
 lu_pivots_to_permutation_p.def_abstract_eval(
     _lu_pivots_to_permutation_abstract_eval)
 batching.primitive_batchers[lu_pivots_to_permutation_p] = (
@@ -1090,7 +1113,7 @@ def _lu_python(x):
   return fn(x)
 
 def _lu_impl(operand):
-  lu, pivot, perm = xla.apply_primitive(lu_p, operand)
+  lu, pivot, perm = dispatch.apply_primitive(lu_p, operand)
   return lu, pivot, perm
 
 def _lu_abstract_eval(operand):
@@ -1191,8 +1214,21 @@ def _lu_cpu_gpu_lowering(getrf_impl, ctx, operand):
   return [lu, pivot, perm]
 
 
-def _lu_tpu_translation_rule(ctx, avals_in, avals_out, operand):
-  return xops.LU(operand)
+def _lu_tpu_lowering_rule(ctx, operand):
+
+  op = hlo.CustomCallOp(
+      [ir.TupleType.get_tuple([mlir.aval_to_ir_type(ctx.avals_out[0]),
+                               mlir.aval_to_ir_type(ctx.avals_out[1]),
+                               mlir.aval_to_ir_type(ctx.avals_out[2])])],
+      [operand],
+      call_target_name=ir.StringAttr.get("LuDecomposition"),
+      has_side_effect=ir.BoolAttr.get(False),
+  )
+  return (
+      hlo.GetTupleElementOp(op, 0).result,
+      hlo.GetTupleElementOp(op, 1).result,
+      hlo.GetTupleElementOp(op, 2).result,
+  )
 
 
 lu_p = Primitive('lu')
@@ -1214,7 +1250,7 @@ mlir.register_lowering(
     lu_p, partial(_lu_cpu_gpu_lowering, gpu_solver.rocm_getrf),
     platform='rocm')
 
-xla.register_translation(lu_p, _lu_tpu_translation_rule, platform='tpu')
+mlir.register_lowering(lu_p, _lu_tpu_lowering_rule, platform='tpu')
 
 
 @partial(vectorize, excluded={3}, signature='(n,n),(n),(n,k)->(n,k)')
@@ -1307,8 +1343,20 @@ def _geqrf_batching_rule(batched_args, batch_dims):
   bd, = batch_dims
   return geqrf(batching.moveaxis(x, bd, 0)), (0, 0)
 
-def _geqrf_translation_rule(ctx, avals_in, avals_out, operand):
-  return xops.QrDecomposition(operand)
+def _geqrf_lowering_rule(ctx, operand):
+  ts_type = mlir.aval_to_ir_type(ctx.avals_out[0])
+  r_type = mlir.aval_to_ir_type(ctx.avals_out[1])
+  op = hlo.CustomCallOp(
+      [ir.TupleType.get_tuple([ts_type, r_type])],
+      [operand],
+      call_target_name=ir.StringAttr.get("Qr"),
+      has_side_effect=ir.BoolAttr.get(False),
+      api_version=mlir.i32_attr(1),
+  )
+  return (
+      hlo.GetTupleElementOp(op, 0).result,
+      hlo.GetTupleElementOp(op, 1).result,
+  )
 
 def _geqrf_cpu_gpu_lowering(geqrf_impl, batched_geqrf_impl, ctx, a):
   if any(not is_constant_shape(a.shape) for a in (ctx.avals_in + ctx.avals_out)):
@@ -1339,10 +1387,10 @@ def _geqrf_cpu_gpu_lowering(geqrf_impl, batched_geqrf_impl, ctx, a):
 
 geqrf_p = Primitive('geqrf')
 geqrf_p.multiple_results = True
-geqrf_p.def_impl(partial(xla.apply_primitive, geqrf_p))
+geqrf_p.def_impl(partial(dispatch.apply_primitive, geqrf_p))
 geqrf_p.def_abstract_eval(_geqrf_abstract_eval)
 batching.primitive_batchers[geqrf_p] = _geqrf_batching_rule
-xla.register_translation(geqrf_p, _geqrf_translation_rule)
+mlir.register_lowering(geqrf_p, _geqrf_lowering_rule)
 
 mlir.register_lowering(
     geqrf_p, partial(_geqrf_cpu_gpu_lowering, lapack.geqrf_hlo, None),
@@ -1398,8 +1446,15 @@ def _householder_product_batching_rule(batched_args, batch_dims):
   return householder_product(batching.moveaxis(a, b_a, 0),
                batching.moveaxis(taus, b_taus, 0)), (0,)
 
-def _householder_product_translation_rule(ctx, avals_in, avals_out, a, taus):
-  return [xops.ProductOfElementaryHouseholderReflectors(a, taus)]
+def _householder_product_lowering_rule(ctx, a, taus):
+  op = hlo.CustomCallOp(
+      [mlir.aval_to_ir_type(ctx.avals_out[0])],
+      [a, taus],
+      call_target_name=ir.StringAttr.get("ProductOfElementaryHouseholderReflectors"),
+      has_side_effect=ir.BoolAttr.get(False),
+      api_version=mlir.i32_attr(1),
+  )
+  return [op.result]
 
 def _householder_product_cpu_gpu_lowering(orgqr_impl, ctx, a, taus):
   if any(not is_constant_shape(a.shape) for a in (ctx.avals_in + ctx.avals_out)):
@@ -1421,10 +1476,10 @@ def _householder_product_cpu_gpu_lowering(orgqr_impl, ctx, a, taus):
 
 
 householder_product_p = Primitive('householder_product')
-householder_product_p.def_impl(partial(xla.apply_primitive, householder_product_p))
+householder_product_p.def_impl(partial(dispatch.apply_primitive, householder_product_p))
 householder_product_p.def_abstract_eval(_householder_product_abstract_eval)
 batching.primitive_batchers[householder_product_p] = _householder_product_batching_rule
-xla.register_translation(householder_product_p, _householder_product_translation_rule)
+mlir.register_lowering(householder_product_p, _householder_product_lowering_rule)
 
 mlir.register_lowering(
     householder_product_p,
@@ -1441,7 +1496,7 @@ mlir.register_lowering(
 
 
 def _qr_impl(operand, *, full_matrices):
-  q, r = xla.apply_primitive(qr_p, operand, full_matrices=full_matrices)
+  q, r = dispatch.apply_primitive(qr_p, operand, full_matrices=full_matrices)
   return q, r
 
 def _qr_abstract_eval(operand, *, full_matrices):
@@ -1519,7 +1574,7 @@ mlir.register_lowering(qr_p, mlir.lower_fun(_qr_lowering));
 # Singular value decomposition
 
 def _svd_impl(operand, *, full_matrices, compute_uv):
-  return xla.apply_primitive(svd_p, operand, full_matrices=full_matrices,
+  return dispatch.apply_primitive(svd_p, operand, full_matrices=full_matrices,
                              compute_uv=compute_uv)
 
 def _svd_abstract_eval(operand, *, full_matrices, compute_uv):
@@ -1709,7 +1764,7 @@ def _tridiagonal_solve_gpu_lowering(lowering, ctx, dl, d, du, b, *, m, n, ldb, t
 tridiagonal_solve_p = Primitive('tridiagonal_solve')
 tridiagonal_solve_p.multiple_results = False
 tridiagonal_solve_p.def_impl(
-    functools.partial(xla.apply_primitive, tridiagonal_solve_p))
+    functools.partial(dispatch.apply_primitive, tridiagonal_solve_p))
 tridiagonal_solve_p.def_abstract_eval(lambda dl, d, du, b, *, m, n, ldb, t: b)
 # TODO(tomhennigan): Consider AD rules using lax.custom_linear_solve?
 
@@ -1820,7 +1875,7 @@ def schur(x: ArrayLike, *,
 
 def _schur_impl(operand, *, compute_schur_vectors, sort_eig_vals,
                 select_callable):
-  return xla.apply_primitive(
+  return dispatch.apply_primitive(
       schur_p,
       operand,
       compute_schur_vectors=compute_schur_vectors,
@@ -1947,7 +2002,7 @@ def _hessenberg_abstract_eval(a):
   return [a, ShapedArray(a.shape[:-2] + (a.shape[-1] - 1,), a.dtype)]
 
 hessenberg_p = Primitive("hessenberg")
-hessenberg_p.def_impl(partial(xla.apply_primitive, hessenberg_p))
+hessenberg_p.def_impl(partial(dispatch.apply_primitive, hessenberg_p))
 hessenberg_p.def_abstract_eval(_hessenberg_abstract_eval)
 hessenberg_p.multiple_results = True
 
@@ -2045,7 +2100,7 @@ def _tridiagonal_abstract_eval(a, *, lower):
   ]
 
 tridiagonal_p = Primitive("tridiagonal")
-tridiagonal_p.def_impl(partial(xla.apply_primitive, tridiagonal_p))
+tridiagonal_p.def_impl(partial(dispatch.apply_primitive, tridiagonal_p))
 tridiagonal_p.def_abstract_eval(_tridiagonal_abstract_eval)
 tridiagonal_p.multiple_results = True
 
