@@ -20,7 +20,6 @@ import weakref
 import numpy as np
 
 import jax
-from jax.interpreters import partial_eval as pe
 
 from jax._src import ad_util
 from jax._src import core
@@ -30,6 +29,7 @@ from jax._src import util
 from jax._src.interpreters import ad
 from jax._src.interpreters import batching
 from jax._src.interpreters import mlir
+from jax._src.interpreters import partial_eval as pe
 from jax._src.lax import lax
 from jax._src.lax.utils import (
     _argnum_weak_type,
@@ -58,8 +58,12 @@ def slice(operand: ArrayLike, start_indices: Sequence[int],
                       limit_indices=tuple(limit_indices),
                       strides=None if strides is None else tuple(strides))
 
-def dynamic_slice(operand: Array, start_indices: Union[Array, Sequence[ArrayLike]],
-                  slice_sizes: Shape) -> Array:
+
+def dynamic_slice(
+    operand: Union[Array, np.ndarray],
+    start_indices: Union[Union[Array, np.ndarray], Sequence[ArrayLike]],
+    slice_sizes: Shape,
+) -> Array:
   """Wraps XLA's `DynamicSlice
   <https://www.tensorflow.org/xla/operation_semantics#dynamicslice>`_
   operator.
@@ -106,7 +110,8 @@ def dynamic_slice(operand: Array, start_indices: Union[Array, Sequence[ArrayLike
   return dynamic_slice_p.bind(operand, *start_indices, *dynamic_sizes,
                               slice_sizes=tuple(static_sizes))
 
-def dynamic_update_slice(operand: Array, update: ArrayLike,
+
+def dynamic_update_slice(operand: Union[Array, np.ndarray], update: ArrayLike,
                          start_indices: Union[Array, Sequence[ArrayLike]]) -> Array:
   """Wraps XLA's `DynamicUpdateSlice
   <https://www.tensorflow.org/xla/operation_semantics#dynamicupdateslice>`_
@@ -635,7 +640,7 @@ def index_take(src: Array, idxs: Array, axes: Sequence[int]) -> Array:
 
 ### convenience wrappers around traceables
 
-def slice_in_dim(operand: Array, start_index: Optional[int],
+def slice_in_dim(operand: Union[Array, np.ndarray], start_index: Optional[int],
                  limit_index: Optional[int],
                  stride: int = 1, axis: int = 0) -> Array:
   """Convenience wrapper around slice applying to only one dimension."""
@@ -664,7 +669,7 @@ def slice_in_dim(operand: Array, start_index: Optional[int],
   return slice(operand, start_indices, limit_indices, strides)
 
 
-def index_in_dim(operand: Array, index: int, axis: int = 0,
+def index_in_dim(operand: Union[Array, np.ndarray], index: int, axis: int = 0,
                  keepdims: bool = True) -> Array:
   """Convenience wrapper around slice to perform int indexing."""
   index, axis = core._canonicalize_dimension(index), int(axis)
@@ -680,7 +685,8 @@ def index_in_dim(operand: Array, index: int, axis: int = 0,
     return lax.squeeze(result, (axis,))
 
 
-def dynamic_slice_in_dim(operand: Array, start_index: ArrayLike,
+def dynamic_slice_in_dim(operand: Union[Array, np.ndarray],
+                         start_index: ArrayLike,
                          slice_size: int, axis: int = 0) -> Array:
   """Convenience wrapper around dynamic_slice applying to one dimension."""
   start_indices: List[ArrayLike] = [lax._const(start_index, 0)] * operand.ndim
@@ -692,8 +698,9 @@ def dynamic_slice_in_dim(operand: Array, start_index: ArrayLike,
   return dynamic_slice(operand, start_indices, slice_sizes)
 
 
-def dynamic_index_in_dim(operand: Array, index: Array, axis: int = 0,
-                         keepdims: bool = True) -> Array:
+def dynamic_index_in_dim(operand: Union[Array, np.ndarray],
+                         index: Union[int, Array],
+                         axis: int = 0, keepdims: bool = True) -> Array:
   """Convenience wrapper around dynamic_slice to perform int indexing."""
   result = dynamic_slice_in_dim(operand, index, 1, axis)
   if keepdims:
@@ -702,7 +709,8 @@ def dynamic_index_in_dim(operand: Array, index: Array, axis: int = 0,
     return lax.squeeze(result, (axis,))
 
 
-def dynamic_update_slice_in_dim(operand: Array, update: ArrayLike,
+def dynamic_update_slice_in_dim(operand: Union[Array, np.ndarray],
+                                update: ArrayLike,
                                 start_index: ArrayLike, axis: int) -> Array:
   """Convenience wrapper around :func:`dynamic_update_slice` to update a slice
      in a single ``axis``.
@@ -713,7 +721,8 @@ def dynamic_update_slice_in_dim(operand: Array, update: ArrayLike,
   return dynamic_update_slice(operand, update, start_indices)
 
 
-def dynamic_update_index_in_dim(operand: Array, update: ArrayLike, index: ArrayLike,
+def dynamic_update_index_in_dim(operand: Union[Array, np.ndarray],
+                                update: ArrayLike, index: ArrayLike,
                                 axis: int) -> Array:
   """Convenience wrapper around :func:`dynamic_update_slice` to update a slice
      of size 1 in a single ``axis``.
@@ -907,7 +916,7 @@ def _dynamic_slice_staging_rule(trace, x, *starts_and_dyn_sizes, slice_sizes):
                                      *starts_and_dyn_sizes,
                                      slice_sizes=slice_sizes)
 
-def _dynamic_slice_typecheck_rule(x, *starts_and_dyn_sizes, slice_sizes):
+def _dynamic_slice_typecheck_rule(_, x, *starts_and_dyn_sizes, slice_sizes):
   start_indices, dyn = util.split_list(starts_and_dyn_sizes, [x.aval.ndim])
   if not dyn:
     out_aval, effects = dynamic_slice_p.abstract_eval(
@@ -1534,13 +1543,10 @@ def _clamp_scatter_indices(operand, indices, updates, *, dnums):
                                    for i in dnums.scatter_dims_to_operand_dims)
   # Stack upper_bounds into a Array[n]
   upper_bound = lax.shape_as_value(upper_bounds)
-  if jax.config.jax_array:
-    # This fix fails lax_test_no_jax_array
-    upper_bound = lax.min(upper_bound,
-                          lax.convert_element_type(np.uint64(np.iinfo(indices.dtype).max),
-                                                   np.int64))
-  else:
-    upper_bound = lax.min(upper_bound, np.iinfo(indices.dtype).max)
+  # This fix fails lax_test_no_jax_array
+  upper_bound = lax.min(upper_bound,
+                        lax.convert_element_type(np.uint64(np.iinfo(indices.dtype).max),
+                                                  np.int64))
 
   upper_bound = lax.broadcast_in_dim(upper_bound, indices.shape,
                                      (len(indices.shape) - 1,))
@@ -1987,7 +1993,6 @@ batching.primitive_batchers[scatter_p] = (
   partial(_scatter_batching_rule, scatter))
 
 
-
 def _scatter_lower(ctx, operand, indices, updates, *,
                    update_jaxpr, update_consts, dimension_numbers,
                    indices_are_sorted, unique_indices, mode):
@@ -2092,8 +2097,8 @@ mlir.register_lowering(scatter_add_p, _scatter_add_lower_gpu, platform="gpu")
 
 
 def _dynamic_slice_indices(
-    operand: Array,
-    start_indices: Union[Array, Sequence[ArrayLike]]
+    operand: Union[Array, np.ndarray],
+    start_indices: Union[Union[Array, np.ndarray], Sequence[ArrayLike]]
   ) -> List[ArrayLike]:
   # Normalize the start_indices w.r.t. operand.shape
   if len(start_indices) != operand.ndim:
