@@ -15,9 +15,14 @@
 """A LazyLoader class."""
 
 import importlib
+from typing import Any, Callable, List, Sequence, Tuple
 
 
-def attach(package_name, submodules):
+def attach(package_name: str, submodules: Sequence[str]) -> Tuple[
+    Callable[[str], Any],
+    Callable[[], List[str]],
+    List[str],
+]:
   """Lazily loads submodules of a package.
 
   Example use:
@@ -26,14 +31,14 @@ def attach(package_name, submodules):
   ```
   """
 
-  __all__ = list(submodules)
+  __all__: List[str] = list(submodules)
 
-  def __getattr__(name):
+  def __getattr__(name: str) -> Any:
     if name in submodules:
       return importlib.import_module(f"{package_name}.{name}")
     raise AttributeError(f"module '{package_name}' has no attribute '{name}")
 
-  def __dir__():
+  def __dir__() -> List[str]:
     return __all__
 
   return __getattr__, __dir__, __all__
