@@ -15,10 +15,9 @@
 # This module is largely a wrapper around `jaxlib` that performs version
 # checking on import.
 
-import platform
+import gc
+import pathlib
 import re
-import os
-import warnings
 from typing import Optional, Tuple
 
 try:
@@ -94,6 +93,11 @@ pytree = xla_client._xla.pytree
 jax_jit = xla_client._xla.jax_jit
 pmap_lib = xla_client._xla.pmap_lib
 
+# XLA garbage collection: see https://github.com/google/jax/issues/14882
+def _xla_gc_callback(*args):
+  xla_client._xla.collect_garbage()
+gc.callbacks.append(_xla_gc_callback)
+
 import jaxlib.gpu_solver as gpu_solver  # pytype: disable=import-error
 import jaxlib.gpu_sparse as gpu_sparse  # pytype: disable=import-error
 import jaxlib.gpu_prng as gpu_prng  # pytype: disable=import-error
@@ -110,16 +114,23 @@ import jaxlib.gpu_rnn as gpu_rnn  # pytype: disable=import-error
 # Version number for MLIR:Python APIs, provided by jaxlib.
 mlir_api_version = xla_client.mlir_api_version
 
-try:
-  from jaxlib import tpu_client as tpu_driver_client  # pytype: disable=import-error
-except:
-  tpu_driver_client = None  # type: ignore
-
-
 # TODO(rocm): check if we need the same for rocm.
-cuda_path: Optional[str]
-cuda_path = os.path.join(os.path.dirname(jaxlib.__file__), "cuda")
-if not os.path.isdir(cuda_path):
-  cuda_path = None
+
+def _cuda_path() -> Optional[str]:
+  _jaxlib_path = pathlib.Path(jaxlib.__file__).parent
+  # If the pip package nvidia-cuda-nvcc-cu11 is installed, it should have
+  # both of the things XLA looks for in the cuda path, namely bin/ptxas and
+  # nvvm/libdevice/libdevice.10.bc
+  path = _jaxlib_path.parent / "nvidia" / "cuda_nvcc"
+  if path.is_dir():
+    return str(path)
+  # Failing that, we use the copy of libdevice.10.bc we include with jaxlib and
+  # hope that the user has ptxas in their PATH.
+  path = _jaxlib_path / "cuda"
+  if path.is_dir():
+    return str(path)
+  return None
+
+cuda_path = _cuda_path()
 
 transfer_guard_lib = xla_client._xla.transfer_guard_lib
