@@ -18,7 +18,7 @@ import jax
 from jax import lax
 from jax import numpy as jnp
 from jax._src.numpy.reductions import _reduction_dims
-from jax._src.numpy.util import _promote_args_inexact
+from jax._src.numpy.util import promote_args_inexact
 from jax._src.typing import Array, ArrayLike
 import numpy as np
 
@@ -67,10 +67,10 @@ def logsumexp(a: ArrayLike, axis: Optional[int] = None, b: Optional[ArrayLike] =
     on the value of the ``return_sign`` argument.
   """
   if b is not None:
-    a_arr, b_arr = _promote_args_inexact("logsumexp", a, b)
+    a_arr, b_arr = promote_args_inexact("logsumexp", a, b)
     a_arr = jnp.where(b_arr != 0, a_arr, -jnp.inf)
   else:
-    a_arr, = _promote_args_inexact("logsumexp", a)
+    a_arr, = promote_args_inexact("logsumexp", a)
     b_arr = a_arr  # for type checking
   pos_dims, dims = _reduction_dims(a_arr, axis)
   amax = jnp.max(a_arr, axis=dims, keepdims=keepdims)
