@@ -17,7 +17,7 @@ import abc
 from functools import partial, reduce
 import math
 import operator as op
-from typing import Any, Callable, Hashable, Iterator, NamedTuple, Sequence
+from typing import Any, Callable, Hashable, Iterator, NamedTuple, Sequence, Union
 
 import numpy as np
 
@@ -42,10 +42,11 @@ from jax._src.lax import lax as lax_internal
 from jax._src.lax import utils as lax_utils
 from jax._src.lib import gpu_prng
 from jax._src.lib.mlir.dialects import hlo
-from jax._src.numpy.lax_numpy import _set_device_array_base_attributes
+from jax._src.numpy.array_methods import _set_device_array_base_attributes
 from jax._src.numpy.util import _register_stackable
-from jax._src.sharding import (
+from jax._src.sharding_impls import (
     NamedSharding, PmapSharding, GSPMDSharding)
+from jax._src.typing import Array
 from jax._src.util import canonicalize_axis, safe_map, safe_zip
 
 map, unsafe_map = safe_map, map
@@ -115,12 +116,12 @@ def _check_prng_key_data(impl, key_data: jax.Array):
 class PRNGKeyArrayMeta(abc.ABCMeta):
   """Metaclass for overriding PRNGKeyArray isinstance checks."""
 
-  def __instancecheck__(self, instance):
+  def __instancecheck__(cls, instance):
     try:
       return (isinstance(instance.aval, core.ShapedArray) and
               type(instance.aval.dtype) is KeyTy)
     except AttributeError:
-      super().__instancecheck__(instance)
+      return super().__instancecheck__(instance)
 
 
 class PRNGKeyArray(metaclass=PRNGKeyArrayMeta):
@@ -265,7 +266,7 @@ basearray.Array.register(PRNGKeyArray)
 
 
 # TODO(frostig): remove, rerouting callers directly to random_seed
-def seed_with_impl(impl: PRNGImpl, seed: int) -> PRNGKeyArray:
+def seed_with_impl(impl: PRNGImpl, seed: Union[int, Array]) -> PRNGKeyArray:
   return random_seed(seed, impl=impl)
 
 
@@ -334,15 +335,7 @@ class KeyTyRules:
   def local_sharded_result_handler(aval, sharding, indices):
     phys_aval, = KeyTyRules.physical_avals(aval)
     key_shape = aval.dtype.impl.key_shape
-
-    # TODO(yashkatariya,frostig): remove this conditional and inline it when
-    # the transient config ever settles
-    if config.jax_array:
-      output_type = pxla.OutputType.Array
-    else:
-      output_type = pxla.OutputType.ShardedDeviceArray
-    phys_handler_maker = pxla.local_result_handlers[
-        (core.ShapedArray, output_type)]
+    phys_handler_maker = pxla.local_result_handlers[core.ShapedArray]
 
     # set up a grounded sharding (with a grounded sharding spec)
     if isinstance(sharding, (PmapSharding, NamedSharding)):
@@ -368,16 +361,7 @@ class KeyTyRules:
   def global_sharded_result_handler(aval, out_sharding, committed,
                                     is_out_sharding_from_xla):
     phys_aval, = KeyTyRules.physical_avals(aval)
-
-    # TODO(yashkatariya,frostig): remove this conditional and inline it when
-    # the transient config ever settles
-    if config.jax_array:
-      output_type = pxla.OutputType.Array
-    else:
-      output_type = pxla.OutputType.GlobalDeviceArray
-
-    phys_handler_maker = pxla.global_result_handlers[
-        (core.ShapedArray, output_type)]
+    phys_handler_maker = pxla.global_result_handlers[core.ShapedArray]
 
     phys_sharding = make_key_array_phys_sharding(
         aval, out_sharding, is_out_sharding_from_xla)
@@ -497,10 +481,6 @@ xla.pytype_aval_mappings[PRNGKeyArray] = (
 
 xla.canonicalize_dtype_handlers[PRNGKeyArray] = lambda x: x
 
-def device_put_key_array(x: PRNGKeyArray, device):
-  return dispatch.device_put(x.unsafe_raw_array(), device)
-dispatch.device_put_handlers[PRNGKeyArray] = device_put_key_array
-
 
 def key_array_shard_arg_handler(x: PRNGKeyArray, devices, indices, sharding):
   # TODO(frostig): Remove the need for `core.get_aval`.
