@@ -24,7 +24,7 @@ import scipy.ndimage
 from jax._src import api
 from jax._src import util
 from jax import lax
-from jax._src.numpy import lax_numpy as jnp
+import jax.numpy as jnp
 from jax._src.numpy.util import _wraps
 from jax._src.typing import ArrayLike, Array
 from jax._src.util import safe_zip as zip
