@@ -18,7 +18,7 @@ import scipy.stats as osp_stats
 from jax import lax
 import jax.numpy as jnp
 from jax._src.lax.lax import _const as _lax_const
-from jax._src.numpy.util import _promote_dtypes_inexact, _wraps
+from jax._src.numpy.util import promote_dtypes_inexact, _wraps
 from jax.scipy.special import gammaln, xlogy
 from jax._src.typing import Array, ArrayLike
 
@@ -30,7 +30,7 @@ def _is_simplex(x: Array) -> Array:
 
 @_wraps(osp_stats.dirichlet.logpdf, update_doc=False)
 def logpdf(x: ArrayLike, alpha: ArrayLike) -> Array:
-  return _logpdf(*_promote_dtypes_inexact(x, alpha))
+  return _logpdf(*promote_dtypes_inexact(x, alpha))
 
 def _logpdf(x: Array, alpha: Array) -> Array:
   if alpha.ndim != 1:
