@@ -11,6 +11,7 @@
 # WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 # See the License for the specific language governing permissions and
 # limitations under the License.
+from __future__ import annotations
 
 import collections
 from dataclasses import dataclass
@@ -427,6 +428,67 @@ def prefix_errors(prefix_tree: Any, full_tree: Any,
                   ) -> List[Callable[[str], ValueError]]:
   return list(_prefix_error((), prefix_tree, full_tree, is_leaf))
 
+def equality_errors(
+    tree1: Any, tree2: Any, is_leaf: Optional[Callable[[Any], bool]] = None,
+) -> Iterable[Tuple[KeyPath, str, str, str]]:
+  yield from _equality_errors((), tree1, tree2, is_leaf)
+
+# TODO(mattjj): maybe share some logic with _prefix_error?
+def _equality_errors(path, t1, t2, is_leaf):
+  # If both are leaves, this isn't a structure equality error.
+  if (treedef_is_strict_leaf(tree_structure(t1, is_leaf=is_leaf)) and
+      treedef_is_strict_leaf(tree_structure(t2, is_leaf=is_leaf))): return
+
+  # The trees may disagree because they are different types:
+  if type(t1) != type(t2):
+    yield path, str(type(t1)), str(type(t2)), 'their Python types differ'
+    return  # no more errors to find
+
+  # Or they may disagree because their roots have different numbers or keys of
+  # children (with special-case handling of list/tuple):
+  if isinstance(t1, (list, tuple)):
+    assert type(t1) == type(t2)
+    if len(t1) != len(t2):
+      yield (path,
+             f'{type(t1).__name__} of length {len(t1)}',
+             f'{type(t2).__name__} of length {len(t2)}',
+             'the lengths do not match')
+      return  # no more errors to find
+  t1_children, t1_meta = flatten_one_level(t1)
+  t2_children, t2_meta = flatten_one_level(t2)
+  t1_keys, t2_keys = _child_keys(t1), _child_keys(t2)
+  try:
+    diff = ' '.join(repr(k.key) for k in
+                    set(t1_keys).symmetric_difference(set(t2_keys)))
+  except:
+    diff = ''
+  if len(t1_children) != len(t2_children):
+    yield (path,
+           f'{type(t1)} with {len(t1_children)} child'
+           f'{"ren" if len(t1_children) > 1 else ""}',
+           f'{type(t2)} with {len(t2_children)} child'
+           f'{"ren" if len(t2_children) > 1 else ""}',
+           'the numbers of children do not match' +
+           (diff and f', with the symmetric difference of key sets: {{{diff}}}')
+           )
+    return  # no more errors to find
+
+  # Or they may disagree if their roots have different pytree metadata:
+  if t1_meta != t2_meta:
+    yield (path,
+           f'{type(t1)} with pytree metadata {t1_meta}',
+           f'{type(t2)} with pytree metadata {t2_meta}',
+           'the pytree node metadata does not match')
+    return  # no more errors to find
+
+  # If the root types and numbers of children agree, there must be a mismatch in
+  # a subtree, so recurse:
+  assert t1_keys == t2_keys, \
+      f"equal pytree nodes gave different tree keys: {t1_keys} and {t2_keys}"
+  for k, c1, c2 in zip(t1_keys, t1_children, t2_children):
+    yield from _equality_errors((*path, k), c1, c2, is_leaf)
+
+
 # TODO(ivyzheng): Remove old APIs when all users migrated.
 
 class _DeprecatedKeyPathEntry(NamedTuple):
@@ -542,10 +604,17 @@ _register_keypaths(
     collections.OrderedDict, lambda x: tuple(DictKey(k) for k in x.keys())
 )
 
+
 def register_pytree_with_keys(
     nodetype: Type[T],
-    flatten_with_keys: Callable[[T], Tuple[Iterable[Tuple[KeyPath, _Children]], _AuxData]],
-    unflatten_func: Callable[[_AuxData, _Children], T]):
+    flatten_with_keys: Callable[
+        [T], Tuple[Iterable[Tuple[KeyEntry, Any]], _AuxData]
+    ],
+    unflatten_func: Callable[[_AuxData, Iterable[Any]], T],
+    flatten_func: Optional[
+        Callable[[T], Tuple[Iterable[Any], _AuxData]]
+    ] = None,
+):
   """Extends the set of types that are considered internal nodes in pytrees.
 
   This is a more powerful alternative to ``register_pytree_node`` that allows
@@ -553,23 +622,32 @@ def register_pytree_with_keys(
 
   Args:
     nodetype: a Python type to treat as an internal pytree node.
-    flatten_func: a function to be used during flattening, taking a value of
-      type ``nodetype`` and returning a pair, with (1) an iterable for tuples of
-      each key path and its child, and (2) some hashable auxiliary data to be
+    flatten_with_keys: a function to be used during flattening, taking a value
+      of type ``nodetype`` and returning a pair, with (1) an iterable for tuples
+      of each key path and its child, and (2) some hashable auxiliary data to be
       stored in the treedef and to be passed to the ``unflatten_func``.
     unflatten_func: a function taking two arguments: the auxiliary data that was
       returned by ``flatten_func`` and stored in the treedef, and the
       unflattened children. The function should return an instance of
       ``nodetype``.
+    flatten_func: an optional function similar to ``flatten_with_keys``, but
+      returns only children and auxiliary data. It must return the children
+      in the same order as ``flatten_with_keys``, and return the same aux data.
+      This argument is optional and only needed for faster traversal when
+      calling functions without keys like ``tree_map`` and ``tree_flatten``.
   """
-  def flatten_func(tree):
-    key_children, treedef = flatten_with_keys(tree)
-    return [c for _, c in key_children], treedef
+  if not flatten_func:
+    def flatten_func_impl(tree):
+      key_children, treedef = flatten_with_keys(tree)
+      return [c for _, c in key_children], treedef
+    flatten_func = flatten_func_impl
+
   register_pytree_node(nodetype, flatten_func, unflatten_func)
   _registry_with_keypaths[nodetype] = _RegistryWithKeypathsEntry(
       flatten_with_keys, unflatten_func
   )
 
+
 def register_pytree_with_keys_class(cls: U) -> U:
   """Extends the set of types that are considered internal nodes in pytrees.
 
@@ -590,8 +668,12 @@ def register_pytree_with_keys_class(cls: U) -> U:
       def tree_unflatten(cls, aux_data, children):
         return cls(*children)
   """
+  flatten_func = (
+      op.methodcaller("tree_flatten") if hasattr(cls, "tree_flatten") else None
+  )
   register_pytree_with_keys(
-      cls, op.methodcaller("tree_flatten_with_keys"), cls.tree_unflatten
+      cls, op.methodcaller("tree_flatten_with_keys"), cls.tree_unflatten,
+      flatten_func
   )
   return cls
 
@@ -613,10 +695,11 @@ def tree_flatten_with_path(
   return _generate_key_paths(tree, is_leaf), tree_def
 
 
-def _generate_key_paths(
+def generate_key_paths(
     tree: Any, is_leaf: Optional[Callable[[Any], bool]] = None
 ) -> List[Tuple[KeyPath, Any]]:
   return list(_generate_key_paths_((), tree, is_leaf))
+_generate_key_paths = generate_key_paths  # alias for backward compat
 
 
 # The overall logic should be same as PyTreeDef::FlattenIntoImpl
@@ -628,18 +711,24 @@ def _generate_key_paths_(
   if is_leaf and is_leaf(tree):
     yield key_path, tree
     return
-  handler = _registry_with_keypaths.get(type(tree))
-  if handler:
-    key_children, _ = handler.flatten_with_keys(tree)
+  key_handler = _registry_with_keypaths.get(type(tree))
+  handler = _registry.get(type(tree))
+  if key_handler:
+    key_children, _ = key_handler.flatten_with_keys(tree)
     for k, c in key_children:
-      yield from _generate_key_paths_(tuple((*key_path, k)), c, is_leaf)
+      yield from _generate_key_paths_((*key_path, k), c, is_leaf)
+  elif handler:
+    children, _ = handler.to_iter(tree)
+    for i, c in enumerate(children):
+      k = FlattenedIndexKey(i)
+      yield from _generate_key_paths_((*key_path, k), c, is_leaf)
   elif isinstance(tree, tuple) and hasattr(tree, '_fields'):
     # handle namedtuple as a special case, based on heuristic
     key_children = [(GetAttrKey(s), getattr(tree, s)) for s in tree._fields]
     for k, c in key_children:
       yield from _generate_key_paths_(tuple((*key_path, k)), c, is_leaf)
-  elif tree is not None:  # Some strictly leaf type, like int or numpy array
-    yield key_path, tree
+  else:
+    yield key_path, tree  # strict leaf type
 
 
 def tree_map_with_path(f: Callable[..., Any],
@@ -773,7 +862,7 @@ def _prefix_error(key_path: KeyPath, prefix_tree: Any, full_tree: Any,
     ("equal pytree nodes gave differing prefix_tree_keys: "
      f"{prefix_tree_keys} and {full_tree_keys}")
   for k, t1, t2 in zip(prefix_tree_keys, prefix_tree_children, full_tree_children):
-    yield from _prefix_error(tuple((*key_path, k)), t1, t2)
+    yield from _prefix_error((*key_path, k), t1, t2)
 
 
 # TODO(jakevdp) remove these deprecated wrappers & their imports in jax/__init__.py
