@@ -29,3 +29,7 @@ from jax._src.custom_derivatives import (
   custom_vjp_call_jaxpr_p as custom_vjp_call_jaxpr_p,
   linear_call as linear_call,
 )
+
+from jax._src.ad_util import (
+  SymbolicZero as SymbolicZero
+)
