@@ -19,11 +19,11 @@ from jax import tree_util
 from jax._src import linear_util as lu
 from jax.experimental import pjit
 from jax.errors import UnexpectedTracerError
+from jax._src import mesh as mesh_lib
 from jax._src.lib.mlir.dialects import hlo
 from jax._src.lib.mlir import ir
-import jax.interpreters.pxla as pxla
 from jax._src.interpreters import mlir
-from jax.interpreters import partial_eval as pe
+from jax._src.interpreters import partial_eval as pe
 from jax._src import custom_api_util
 from jax._src.lib import xla_client as xc
 from jax._src.api_util import flatten_fun_nokwargs
@@ -209,7 +209,7 @@ class custom_partitioning:
     .. code-block:: python
 
       import jax
-      from jax._src.sharding import NamedSharding
+      from jax.sharding import NamedSharding
       from jax.experimental.custom_partitioning import custom_partitioning
       from jax.experimental.pjit import pjit
       from jax.sharding import PartitionSpec as P
@@ -370,7 +370,7 @@ def _custom_partitioning_lowering_rule(ctx: mlir.LoweringRuleContext, *values,
                                        propagate_user_sharding, partition,
                                        infer_sharding_from_operands,
                                        static_args):
-  mesh = pxla.thread_resources.env.physical_mesh
+  mesh = mesh_lib.thread_resources.env.physical_mesh
   axis_context = ctx.module_context.axis_context
 
   if isinstance(axis_context, mlir.ShardingContext):
@@ -386,7 +386,8 @@ def _custom_partitioning_lowering_rule(ctx: mlir.LoweringRuleContext, *values,
 
   def to_mesh_pspec_sharding(op_sharding: xc.OpSharding):
     if mesh.empty:
-      from jax._src.sharding import GSPMDSharding
+      from jax._src.sharding_impls import GSPMDSharding
+      assert devices is not None
       return GSPMDSharding(devices, op_sharding)
     pspec = pjit.parse_flatten_op_sharding(op_sharding,
                                            mesh)[0].get_partition_spec()
