@@ -26,9 +26,9 @@ from typing import Callable, Sequence, Optional, Dict, Any
 import jax
 from jax._src import distributed
 from jax._src.config import config
-from jax._src import global_device_array as gda
 from jax._src import array
 from jax._src import sharding
+from jax._src import sharding_impls
 from jax._src import typing
 from jax.sharding import Mesh
 import jax.numpy as jnp
@@ -46,7 +46,7 @@ logger = logging.getLogger(__name__)
 
 async def create_async_array_from_callback(
     global_shape: array.Shape,
-    inp_sharding: sharding.XLACompatibleSharding,
+    inp_sharding: sharding_impls.XLACompatibleSharding,
     data_callback: Callable[[array.Index], asyncio.Future],
 ):
   device_to_index_map = inp_sharding.devices_indices_map(global_shape)
@@ -63,27 +63,6 @@ async def create_async_array_from_callback(
       global_shape, inp_sharding, dbs)
 
 
-async def create_async_gda_from_callback(
-    global_shape: gda.Shape,
-    global_mesh: Mesh,
-    mesh_axes: gda.MeshAxes,
-    data_callback: Callable[[gda.Index], asyncio.Future],
-):
-  global_idx_rid = gda.get_shard_indices_replica_ids(
-      global_shape, global_mesh, mesh_axes)
-  local_devices = global_mesh.local_devices
-  future_arrays = [data_callback(global_idx_rid[d][0])
-                   for d in local_devices]
-  # Pause here and come back to `from_async_callback()` when future_arrays are
-  # ready. device_put cannot happen with future_arrays.
-  local_arrays = await asyncio.gather(*future_arrays)
-
-  dbs = [jax.device_put(array, device)
-         for array, device in zip(local_arrays, local_devices)]
-  return gda.GlobalDeviceArray(global_shape, global_mesh, mesh_axes, dbs,
-                               gda._GdaFastPathArgs(global_idx_rid, local_devices))
-
-
 def _get_metadata(arr):
   if arr.dtype == jnp.bfloat16:
     # Tensorstore uses 'bfloat16', not '<V2'.
@@ -163,7 +142,9 @@ class _LimitInFlightBytes:
       self._cv.notify_all()
 
 
-async def async_serialize(arr_inp, tensorstore_spec, commit_future=None):
+async def async_serialize(
+    arr_inp, tensorstore_spec, commit_future=None, context=TS_CONTEXT
+):
   if (isinstance(arr_inp, array.ArrayImpl) and jax.process_count() > 1 and
       arr_inp.is_fully_addressable):
     raise ValueError('Passing fully addressable Arrays to a multiprocess '
@@ -175,7 +156,11 @@ async def async_serialize(arr_inp, tensorstore_spec, commit_future=None):
 
   if jax.process_index() == 0:
     open_future = ts.open(
-        ts.Spec(tensorstore_spec), create=True, open=True, context=TS_CONTEXT)
+        ts.Spec(tensorstore_spec),
+        create=True,
+        open=True,
+        context=context,
+    )
     # Asynchronous case.
     if commit_future is not None:
       assert isinstance(commit_future, list)
@@ -189,7 +174,11 @@ async def async_serialize(arr_inp, tensorstore_spec, commit_future=None):
   # tensorstore object.
   # For every process other than `0`, we open with `assume_metadata=True`.
   t = await ts.open(
-      ts.Spec(tensorstore_spec), open=True, assume_metadata=True, context=TS_CONTEXT)
+      ts.Spec(tensorstore_spec),
+      open=True,
+      assume_metadata=True,
+      context=context,
+  )
 
   async def _write_array(shard):
     if shard.replica_id == 0:
@@ -246,10 +235,15 @@ def estimate_read_memory_footprint(t: ts.TensorStore) -> int:
   return num_bytes
 
 
-async def async_deserialize(in_sharding, tensorstore_spec,
-                            global_shape=None, dtype=None,
-                            byte_limiter: Optional[_LimitInFlightBytes] = None):
-  t = await ts.open(ts.Spec(tensorstore_spec), open=True, context=TS_CONTEXT)
+async def async_deserialize(
+    in_sharding,
+    tensorstore_spec,
+    global_shape=None,
+    dtype=None,
+    byte_limiter: Optional[_LimitInFlightBytes] = None,
+    context=TS_CONTEXT,
+):
+  t = await ts.open(ts.Spec(tensorstore_spec), open=True, context=context)
   shape = t.shape if global_shape is None else global_shape
   new_shard_shape = in_sharding.shard_shape(tuple(shape))
 
@@ -279,15 +273,7 @@ async def async_deserialize(in_sharding, tensorstore_spec,
       await byte_limiter.release_bytes(requested_bytes)
     return out
 
-  if config.jax_array:
-    return await create_async_array_from_callback(tuple(shape), in_sharding, cb)
-  else:
-    if not isinstance(in_sharding, sharding.NamedSharding):
-      raise ValueError('Deserializing a GlobalDeviceArray is only possible with '
-                       'a `NamedSharding` which consists of a `mesh` and '
-                       f'`pspec`, but got {in_sharding}')
-    return await create_async_gda_from_callback(
-        tuple(shape), in_sharding.mesh, in_sharding.spec, cb)
+  return await create_async_array_from_callback(tuple(shape), in_sharding, cb)
 
 
 def run_deserialization(shardings: Sequence[sharding.Sharding],
