@@ -12,10 +12,12 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 """Tests for call_tf."""
+import base64
 from functools import partial
 from typing import Callable, Dict, Tuple
 import unittest
 
+from absl import logging
 from absl.testing import absltest
 from absl.testing import parameterized
 
@@ -27,6 +29,8 @@ from jax._src import test_util as jtu
 from jax.config import config
 from jax.experimental import jax2tf
 from jax.experimental.jax2tf.tests import tf_test_util
+from jax._src.lib.mlir import ir
+from tensorflow.core.framework import function_pb2
 
 import numpy as np
 
@@ -58,7 +62,7 @@ _parameterized_jit = parameterized.named_parameters(
     _named_test(with_jit=with_jit)
     for with_jit in [True, False])
 
-_call_tf_non_compileable_error = "Error compiling TensorFlow function. call_tf can used in a staged context .* only with compileable functions"
+_call_tf_non_compilable_error = "Error compiling TensorFlow function. call_tf can used in a staged context .* only with compilable functions"
 _call_tf_dynamic_shape_error = "call_tf cannot call functions whose output has dynamic shape"
 
 class CallTfTest(tf_test_util.JaxToTfTestCase):
@@ -151,37 +155,37 @@ class CallTfTest(tf_test_util.JaxToTfTestCase):
     res = fun_jax()
     self.assertAllClose(res, (x1, x2))
 
-  def test_error_non_compileable_strings(self):
+  def test_error_non_compilable_strings(self):
     # Check that in op-by-op we call a function in eager mode.
-    def f_tf_non_compileable(x):
+    def f_tf_non_compilable(x):
       return tf.strings.length(tf.strings.format("Hello {}!", [x]))
 
-    f_jax = jax2tf.call_tf(f_tf_non_compileable)
+    f_jax = jax2tf.call_tf(f_tf_non_compilable)
     x = np.float32(0.7)
-    self.assertAllClose(f_tf_non_compileable(x).numpy(), f_jax(x))
+    self.assertAllClose(f_tf_non_compilable(x).numpy(), f_jax(x))
     with self.assertRaisesRegex(ValueError,
-                                _call_tf_non_compileable_error):
+                                _call_tf_non_compilable_error):
       jax.jit(f_jax)(x)
 
     with self.assertRaisesRegex(ValueError,
-                                _call_tf_non_compileable_error):
+                                _call_tf_non_compilable_error):
       lax.cond(True, lambda x: f_jax(x), lambda x: f_jax(x), x)
 
-  def test_error_non_compileable_dynamic_shape(self):
+  def test_error_non_compilable_dynamic_shape(self):
     # Check that in op-by-op we call a function in eager mode.
-    def f_tf_non_compileable(x):
+    def f_tf_non_compilable(x):
       return tf.cond(x[0], lambda: x[1:], lambda: x)
 
-    f_jax = jax2tf.call_tf(f_tf_non_compileable)
+    f_jax = jax2tf.call_tf(f_tf_non_compilable)
     x = np.array([True, False], dtype=np.bool_)
-    self.assertAllClose(f_tf_non_compileable(x), f_jax(x))  # Works in eager mode
+    self.assertAllClose(f_tf_non_compilable(x), f_jax(x))  # Works in eager mode
 
     with self.assertRaisesRegex(ValueError, _call_tf_dynamic_shape_error):
       jax.jit(f_jax)(x)
 
   def test_error_bad_result_tensorarray(self):
     # Call a function that returns a tf.TensorArray. This should be detected
-    # early on. If we don't the function is actually compileable but returns
+    # early on. If we don't the function is actually compilable but returns
     # a tuple instead of a single result.
     def fun_tf():
       ta = tf.TensorArray(tf.int32, size=0, dynamic_size=True)
@@ -197,7 +201,7 @@ class CallTfTest(tf_test_util.JaxToTfTestCase):
     def fun_tf():
       return tf.constant("foo")
 
-    # Now under jit, should fail because the function is not compileable
+    # Now under jit, should fail because the function is not compilable
     with self.assertRaisesRegex(ValueError,
                                 "The called TF function returns a result that is not convertible to JAX"):
       fun_jax = jax.jit(jax2tf.call_tf(fun_tf))
@@ -541,11 +545,11 @@ class CallTfTest(tf_test_util.JaxToTfTestCase):
       grad_jax_pure = jax.grad(grad_jax_pure)
 
     res_jax = grad_jax(np.float32(5.))
-    print(f"Grad of {degree} degree is {res_jax}")
+    logging.info("Grad of %s degree is %s", degree, res_jax)
     self.assertAllClose(res_jax, grad_jax_pure(np.float32(5.)))
 
   def test_pmap(self):
-    print(f"Running test_pmap on {jax.local_device_count()} devices")
+    logging.info("Running test_pmap on %s devices", jax.local_device_count())
 
     def plus_2_tf(x):
       return tf.math.add(2., x)
@@ -572,7 +576,7 @@ class CallTfTest(tf_test_util.JaxToTfTestCase):
     res1 = jax2tf.call_tf(fun_tf)(x)
     self.assertAllClose(x[0:x[1]], res1)
 
-    # Now under jit, should fail because the function is not compileable
+    # Now under jit, should fail because the function is not compilable
     with self.assertRaisesRegex(ValueError, _call_tf_dynamic_shape_error):
       fun_jax = jax.jit(jax2tf.call_tf(fun_tf))
       fun_jax(x)
@@ -668,9 +672,6 @@ class CallTfTest(tf_test_util.JaxToTfTestCase):
     self.assertAllClose(res.numpy(), f_jax(x))
 
   def test_effectful(self):
-    if not config.jax_array:
-      raise unittest.SkipTest("Test not intended to work without jax.Array")
-
     x = np.ones((3,), dtype=np.float32)
     lower_effect = jax.jit(jax2tf.call_tf(tf.math.sin, has_side_effects=True)).lower(x)
     self.assertNotEmpty(lower_effect._lowering.compile_args["unordered_effects"])
@@ -698,8 +699,8 @@ class CallTfTest(tf_test_util.JaxToTfTestCase):
     # Uses TF gradient for `cos_tf` and JAX gradient for `sin`
     jax.grad(cos_tf_sin_jax)(x)
 
-    print(jax.make_jaxpr(cos_tf_sin_jax)(x))
-    print(jax.xla_computation(cos_tf_sin_jax)(x).as_hlo_text())
+    logging.info(jax.make_jaxpr(cos_tf_sin_jax)(x))
+    logging.info(jax.xla_computation(cos_tf_sin_jax)(x).as_hlo_text())
 
   def test_tf_gather(self):
     """tf_gather gradient output is tf.IndexSlices."""
@@ -886,7 +887,7 @@ class RoundTripToJaxTest(tf_test_util.JaxToTfTestCase):
     y = np.array([-0.5, 0.0, 0.5], dtype=np.float32)
 
     converted_fun = tf.function(
-        jax2tf.convert(fun_jax, experimental_native_lowering=True)
+        jax2tf.convert(fun_jax, native_serialization=True)
     )
     expected = np.sin(x) + np.cos(y)
     res = tf.function(converted_fun, jit_compile=True, autograph=False)(x, y)
@@ -1100,7 +1101,7 @@ class RoundTripToTfTest(tf_test_util.JaxToTfTestCase):
     expected = x[1:]
     self.assertAllClose(expected, res1, check_dtypes=False)
 
-    # Now under jit, should fail because the function is not compileable
+    # Now under jit, should fail because the function is not compilable
     with self.assertRaisesRegex(ValueError, _call_tf_dynamic_shape_error):
       fun_jax = jax.jit(jax2tf.call_tf(fun_tf))
       fun_jax(x)
@@ -1113,8 +1114,8 @@ class RoundTripToTfTest(tf_test_util.JaxToTfTestCase):
 
   @_parameterized_jit
   def test_shape_poly_static_output_shape(self, with_jit=True):
-    if config.jax2tf_default_experimental_native_lowering:
-      raise unittest.SkipTest("TODO(b/268386622): call_tf with shape polymorphism and native lowering.")
+    if config.jax2tf_default_native_serialization:
+      raise unittest.SkipTest("TODO(b/268386622): call_tf with shape polymorphism and native serialization.")
     x = np.array([0.7, 0.8], dtype=np.float32)
 
     def fun_tf(x):
@@ -1127,8 +1128,8 @@ class RoundTripToTfTest(tf_test_util.JaxToTfTestCase):
 
   @_parameterized_jit
   def test_shape_poly(self, with_jit=False):
-    if config.jax2tf_default_experimental_native_lowering:
-      raise unittest.SkipTest("TODO(b/268386622): call_tf with shape polymorphism and native lowering.")
+    if config.jax2tf_default_native_serialization:
+      raise unittest.SkipTest("TODO(b/268386622): call_tf with shape polymorphism and native serialization.")
     x = np.array([7, 8, 9, 10], dtype=np.float32)
     def fun_jax(x):
       y = jax2tf.call_tf(tf.math.sin,
@@ -1146,8 +1147,8 @@ class RoundTripToTfTest(tf_test_util.JaxToTfTestCase):
 
   @_parameterized_jit
   def test_shape_poly_pytree_result(self, with_jit=True):
-    if config.jax2tf_default_experimental_native_lowering:
-      raise unittest.SkipTest("TODO(b/268386622): call_tf with shape polymorphism and native lowering.")
+    if config.jax2tf_default_native_serialization:
+      raise unittest.SkipTest("TODO(b/268386622): call_tf with shape polymorphism and native serialization.")
     x = np.array([7, 8, 9, 10], dtype=np.float32)
     def fun_jax(x):
       # Returns a tuple
@@ -1232,25 +1233,25 @@ class RoundTripToTfTest(tf_test_util.JaxToTfTestCase):
     if kind == "bad_dim" and with_jit:
       # TODO: in jit more the error pops up later, at AddV2
       expect_error = "Dimensions must be equal, but are 4 and 9 for .* AddV2"
-    if kind == "bad_dim" and config.jax2tf_default_experimental_native_lowering:
-      # TODO(b/268386622): call_tf with shape polymorphism and native lowering.
-      expect_error = "Error compiling TensorFlow function. call_tf can used .* only with compileable functions with static output shapes"
+    if kind == "bad_dim" and config.jax2tf_default_native_serialization:
+      # TODO(b/268386622): call_tf with shape polymorphism and native serialization.
+      expect_error = "Error compiling TensorFlow function. call_tf can used .* only with compilable functions with static output shapes"
     fun_tf_rt = _maybe_tf_jit(with_jit,
         jax2tf.convert(fun_jax, polymorphic_shapes=["b, ..."]))
     with self.assertRaisesRegex(expect_ex, expect_error):
       fun_tf_rt(x)
 
-  def test_inner_native_lowering(self):
-    # Two nested jax2tf, the inner one being with native lowering
+  def test_inner_native_serialization(self):
+    # Two nested jax2tf, the inner one being with native serialization
     x = np.ones((3,), dtype=np.float32)
     def f_inner_jax(x):
       return jnp.sin(x)
     def f_outer_jax(x):
-      f_inner_tf = jax2tf.convert(f_inner_jax, experimental_native_lowering=True)
+      f_inner_tf = jax2tf.convert(f_inner_jax, native_serialization=True)
       return jnp.cos(jax2tf.call_tf(f_inner_tf)(x))
 
     f_outer_tf = tf.function(
-        jax2tf.convert(f_outer_jax, experimental_native_lowering=False),
+        jax2tf.convert(f_outer_jax, native_serialization=False),
         autograph=False)
     f_outer_graph = str(f_outer_tf.get_concrete_function(tf.convert_to_tensor(x)).graph.as_graph_def())
     # Quick way to check that there is an XlaCallModule op, and a Cos op, but no Sin op
@@ -1306,6 +1307,131 @@ class RoundTripToTfTest(tf_test_util.JaxToTfTestCase):
     _, (g_f4_ft,) = tf_test_util.ComputeTfValueAndGrad(f4_tf, [x])
     self.assertAllClose(jax.grad(f(4))(x), g_f4_ft.numpy())
 
+  @classmethod
+  def _walk_stablehlo_operations(cls, op, cb):
+    """walk the stablehlo operation recursive with callback function."""
+    cb(op)
+    for region in op.operation.regions:
+      for block in region:
+        for op in block:
+          cls._walk_stablehlo_operations(op, cb)
+
+  def test_use_custom_call(self):
+    const = tf.Variable(0.0, dtype=tf.float32)
+
+    @tf.function(jit_compile=True)
+    def tf_func_1(x):
+      return x * x + const
+
+    @tf.function
+    def tf_func_2(x, y):
+      return tf_func_1(x) + y
+
+    @tf.function
+    def tf_func_3(x, y, z):
+      return tf_func_2(x, y) + z, z
+
+    x = jnp.array(3.0, dtype=jnp.float32)
+    y = jnp.array(3.0, dtype=jnp.float32)
+    z = jnp.array(5.0, dtype=jnp.float32)
+    output_shape_dtype = (
+        jax.ShapeDtypeStruct(x.shape, x.dtype),
+        jax.ShapeDtypeStruct(z.shape, z.dtype),
+    )
+    f_jax = jax.jit(jax2tf.call_tf(tf_func_3, use_custom_call=False))
+    stablehlo_module = f_jax.lower(x, y, z).compiler_ir("stablehlo")
+    self.assertNotIn("stablehlo.custom_call", str(stablehlo_module))
+
+    f_jax = jax.jit(
+        jax2tf.call_tf(
+            tf_func_3,
+            use_custom_call=True,
+            output_shape_dtype=output_shape_dtype,
+        )
+    )
+    stablehlo_module = f_jax.lower(x, y, z).compiler_ir("stablehlo")
+    self.assertIn("stablehlo.custom_call", str(stablehlo_module))
+
+    concrete_function_flat_tf = tf_func_3.get_concrete_function(x, y, z)
+    expect_function_def_dict = {}
+    expect_function_def_dict[
+        concrete_function_flat_tf.function_def.signature.name
+    ] = concrete_function_flat_tf.function_def
+    for k, v in concrete_function_flat_tf.graph._functions.items():
+      expect_function_def_dict[k] = v.definition
+
+    deserialized_function_def_dict = {}
+
+    def extract_func_def(op):
+      if op.operation.name != "stablehlo.custom_call":
+        return
+      tf_metadata = ir.DictAttr(op.attributes["tf_metadata"])
+      function_def_list = ir.ArrayAttr(tf_metadata["function_def_list"])
+
+      for fdef_str in function_def_list:
+        fdef_str_bytes = base64.b64decode(str(fdef_str)[1:-1])
+        fdef = function_pb2.FunctionDef()
+        fdef.ParseFromString(fdef_str_bytes)
+        deserialized_function_def_dict.update({fdef.signature.name: fdef})
+
+    self._walk_stablehlo_operations(stablehlo_module, extract_func_def)
+
+    for k, _ in expect_function_def_dict.items():
+      self.assertEqual(
+          expect_function_def_dict[k], deserialized_function_def_dict[k]
+      )
+
+  def test_use_custom_call_non_compilable(self):
+    deserialized_function_def_dict = {}
+
+    def extract_func_def(op):
+      if op.operation.name != "stablehlo.custom_call":
+        return
+      tf_metadata = ir.DictAttr(op.attributes["tf_metadata"])
+      function_def_list = ir.ArrayAttr(tf_metadata["function_def_list"])
+
+      for fdef_str in function_def_list:
+        fdef_str_bytes = base64.b64decode(str(fdef_str)[1:-1])
+        fdef = function_pb2.FunctionDef()
+        fdef.ParseFromString(fdef_str_bytes)
+        deserialized_function_def_dict.update({fdef.signature.name: fdef})
+
+    @tf.function(jit_compile=False)
+    def my_op(x):
+      return tf.py_function(np.sin, [x], tf.float32)
+
+    x = jnp.ones([10], dtype=jnp.float32)
+    output_shape_dtype = jax.ShapeDtypeStruct(x.shape, x.dtype)
+    f_jax = jax.jit(
+        jax2tf.call_tf(
+            my_op,
+            use_custom_call=False,
+            output_shape_dtype=output_shape_dtype,
+        )
+    )
+
+    f_jax = jax.jit(
+        jax2tf.call_tf(
+            my_op,
+            use_custom_call=True,
+            output_shape_dtype=output_shape_dtype,
+        )
+    )
+    stablehlo_module = f_jax.lower(x).compiler_ir("stablehlo")
+    concrete_function_flat_tf = my_op.get_concrete_function(x)
+    expect_function_def_dict = {}
+    expect_function_def_dict[
+        concrete_function_flat_tf.function_def.signature.name
+    ] = concrete_function_flat_tf.function_def
+    for k, v in concrete_function_flat_tf.graph._functions.items():
+      expect_function_def_dict[k] = v.definition
+
+    self._walk_stablehlo_operations(stablehlo_module, extract_func_def)
+    for k, _ in expect_function_def_dict.items():
+      self.assertEqual(
+          expect_function_def_dict[k], deserialized_function_def_dict[k]
+      )
+
 
 if __name__ == "__main__":
   absltest.main(testLoader=jtu.JaxTestLoader())
