@@ -17,6 +17,7 @@ Specific JAX primitive conversion tests are in primitives_test."""
 import collections
 import contextlib
 import os
+import re
 from typing import Callable, Dict, Optional, Tuple
 import unittest
 
@@ -29,11 +30,13 @@ from jax import dtypes
 from jax import lax
 from jax import numpy as jnp
 from jax import sharding
+from jax._src import core
 from jax._src import source_info_util
 from jax._src import test_util as jtu
 import jax._src.xla_bridge
 from jax.config import config
 from jax.experimental import jax2tf
+from jax.experimental.jax2tf import jax_export
 from jax.experimental.jax2tf.tests import tf_test_util
 from jax.experimental.maps import xmap
 from jax.experimental.shard_map import shard_map
@@ -56,6 +59,28 @@ class Jax2TfTest(tf_test_util.JaxToTfTestCase):
     f_jax = lambda x, y: x
     self.ConvertAndCompare(f_jax, 0.7, 1)
 
+  def test_sin(self):
+    f_tf = jax2tf.convert(jnp.sin)
+    x = np.float32(.5)
+    sin_x = np.sin(x)
+    self.assertAllClose(sin_x, f_tf(x))
+    self.assertAllClose(sin_x, tf.function(f_tf, autograph=False,
+                                           jit_compile=True)(x))
+    # TODO: The following, with jit_compile=False, fails with
+    # native serialization because the tf.function() somehow executes the
+    # XlaCallModule op on CPU. This is despite the `with tf.device()`
+    # tf_preferred_device = (
+    #     tf.config.list_logical_devices("TPU") +
+    #     tf.config.list_logical_devices("GPU") +
+    #     tf.config.list_logical_devices())[0]
+    # logging.info("Running TF on %s", tf_preferred_device)
+    # with tf.device(tf_preferred_device):
+    #   self.assertAllClose(sin_x, tf.function(f_tf, autograph=False,
+    #                                         jit_compile=False)(x))
+
+    # self.assertAllClose(sin_x, tf.function(f_tf, autograph=False,
+    #                                       jit_compile=False)(x))
+
   def test_basics(self):
     f_jax = lambda x: jnp.sin(jnp.cos(x))
     self.ConvertAndCompare(f_jax, 0.7)
@@ -802,9 +827,9 @@ class Jax2TfTest(tf_test_util.JaxToTfTestCase):
     def f_jax():
       return jnp.sin(1.)
     f_tf = jax2tf.convert(f_jax)
-    # for native lowering the HLO we get from TF is constant-folded, so this
+    # for native serialization the HLO we get from TF is constant-folded, so this
     # test fails.
-    if not config.jax2tf_default_experimental_native_lowering:
+    if not config.jax2tf_default_native_serialization:
       self.assertIn("sine(", self.TfToHlo(f_tf))
 
   def test_convert_of_nested_independent_jit(self):
@@ -894,7 +919,7 @@ class Jax2TfTest(tf_test_util.JaxToTfTestCase):
 
       out = jax2tf.convert(caller_jax, with_gradient=False)(2.)
       return out
-    if config.jax2tf_default_experimental_native_lowering:
+    if config.jax2tf_default_native_serialization:
       self.assertIn("my_test_function_jax/mul", self.TfToHlo(run_tf))
     else:
       graph_def = str(tf.function(run_tf, autograph=False).get_concrete_function().graph.as_graph_def())
@@ -924,8 +949,8 @@ class Jax2TfTest(tf_test_util.JaxToTfTestCase):
   def test_shared_constants(self):
     # Check that the constants are shared properly in converted functions
     # See https://github.com/google/jax/issues/7992.
-    if config.jax2tf_default_experimental_native_lowering:
-      raise unittest.SkipTest("shared constants tests not interesting for native lowering")
+    if config.jax2tf_default_native_serialization:
+      raise unittest.SkipTest("shared constants tests not interesting for native serialization")
     const = np.random.uniform(size=256).astype(np.float32)  # A shared constant
     def f(x):
       return x + const + const + const + const
@@ -936,8 +961,8 @@ class Jax2TfTest(tf_test_util.JaxToTfTestCase):
   def test_shared_constants_under_cond(self):
     # Check that the constants are shared properly in converted functions
     # See https://github.com/google/jax/issues/7992.
-    if config.jax2tf_default_experimental_native_lowering:
-      raise unittest.SkipTest("shared constants tests not interesting for native lowering")
+    if config.jax2tf_default_native_serialization:
+      raise unittest.SkipTest("shared constants tests not interesting for native serialization")
     const_size = 512
     const = np.random.uniform(size=const_size).astype(np.float32)  # A shared constant
     x = np.ones((const_size,), dtype=np.float32)
@@ -952,8 +977,8 @@ class Jax2TfTest(tf_test_util.JaxToTfTestCase):
 
   def test_shared_constants_under_scan(self):
     # See https://github.com/google/jax/issues/7992.
-    if config.jax2tf_default_experimental_native_lowering:
-      raise unittest.SkipTest("shared constants tests not interesting for native lowering")
+    if config.jax2tf_default_native_serialization:
+      raise unittest.SkipTest("shared constants tests not interesting for native serialization")
     const_size = 512
     const = np.random.uniform(size=const_size).astype(np.float32)  # A shared constant
     xs = np.ones((8, const_size), dtype=np.float32)
@@ -971,8 +996,8 @@ class Jax2TfTest(tf_test_util.JaxToTfTestCase):
 
   def test_shared_constants_under_jit(self):
     # We do not share constants under jit.
-    if config.jax2tf_default_experimental_native_lowering:
-      raise unittest.SkipTest("shared constants tests not interesting for native lowering")
+    if config.jax2tf_default_native_serialization:
+      raise unittest.SkipTest("shared constants tests not interesting for native serialization")
     const = np.random.uniform(size=(16, 16)).astype(np.float32)  # A shared constant
     @jax.jit
     def g_jit(x):
@@ -987,8 +1012,8 @@ class Jax2TfTest(tf_test_util.JaxToTfTestCase):
     # randint has the property that the TF lowering of the randbits_p
     # primitive generates constants that did not exist in the Jaxpr. As such
     # it has created new errors related to the sharing of the constants.
-    if config.jax2tf_default_experimental_native_lowering:
-      raise unittest.SkipTest("shared constants tests not interesting for native lowering")
+    if config.jax2tf_default_native_serialization:
+      raise unittest.SkipTest("shared constants tests not interesting for native serialization")
 
     key = jax.random.PRNGKey(42)
 
@@ -1054,7 +1079,7 @@ class Jax2TfTest(tf_test_util.JaxToTfTestCase):
                         (grad_tf[0].numpy(), grad_tf[1].numpy()))
 
 
-  @jtu.skip_on_flag("jax2tf_default_experimental_native_lowering", True)
+  @jtu.skip_on_flag("jax2tf_default_native_serialization", True)
   def test_enable_xla(self):
     # Tests that enable_xla flag is properly scoped to a conversion.
     def fun(x):
@@ -1087,13 +1112,19 @@ class Jax2TfTest(tf_test_util.JaxToTfTestCase):
       return jax.random.randint(
           jax.random.PRNGKey(42), shape=(), minval=0, maxval=1)
 
-    with contextlib.ExitStack() as stack:
-      if (jtu.device_under_test() == "gpu" and
-          config.jax2tf_default_experimental_native_lowering):
-        stack.enter_context(
-            self.assertRaisesRegex(ValueError,
-                "Cannot serialize code with custom calls whose targets .*"))
-      self.ConvertAndCompare(randint)
+    self.ConvertAndCompare(randint)
+
+  def test_error_disallowed_custom_call(self):
+    if jtu.device_under_test() != "cpu":
+      self.skipTest("Test intended for CPU only")
+    # For now triangular_solve on CPU uses the unsupported "blas_strsm" target
+    a = np.arange(16, dtype=np.float32).reshape((4, 4))
+    b = np.arange(4, dtype=np.float32).reshape((4, 1))
+    with self.assertRaisesRegex(ValueError,
+        "Cannot serialize code with custom calls whose targets .*"):
+      jax2tf.convert(
+          lambda a, b: jax.lax.linalg.triangular_solve(a, b, left_side=True),
+          native_serialization=True)(a, b)
 
   def test_op_metadata_simple(self):
     self.skipTest("include_xla_op_metadata not yet enabled")
@@ -1289,7 +1320,7 @@ class Jax2TfTest(tf_test_util.JaxToTfTestCase):
         self.fail(f"{op.name} does not start with {scope_name}.")
 
   def test_name_scope_polymorphic(self):
-    if config.jax2tf_default_experimental_native_lowering and not config.jax_dynamic_shapes:
+    if config.jax2tf_default_native_serialization and not config.jax_dynamic_shapes:
       self.skipTest("shape polymorphism but --jax_dynamic_shapes is not set.")
 
     def func_jax(x, y):
@@ -1407,15 +1438,11 @@ class Jax2TfTest(tf_test_util.JaxToTfTestCase):
                      transform2 != "none") else
           [False, True])
   )
-  def test_cross_platform(self, with_mesh=True, transform1="xmap",
-                          transform2="none", nullary=True):
+  def test_cross_platform(self, with_mesh=True, transform1="pjit_in_shardings_P",
+                          transform2="pjit_in_shardings_P", nullary=False):
     # Tests cross-lowering for
     #  with mesh:
     #   transform2(transform1(func))
-    if not config.jax_array:
-      raise unittest.SkipTest("cross_platform test work only with jax.Array")
-    if not config.jax_jit_pjit_api_merge:
-      raise unittest.SkipTest("cross_platform test work only with jit==pjit")
     if transform2 == "none" and (
         transform1 == "shard_map" or
         transform1 in ["pjit_in_shardings_P", "pjit_in_shardings_Sharding"] and nullary):
@@ -1481,20 +1508,84 @@ class Jax2TfTest(tf_test_util.JaxToTfTestCase):
       raise unittest.SkipTest("TODO: figure out how to invoke pmap from TF")
 
     f_tf = jax2tf.convert(func_to_convert,
-                          experimental_native_lowering=True,
-                          experimental_native_lowering_platforms=('tpu',))
+                          native_serialization=True,
+                          native_serialization_platforms=('tpu',))
     f_tf = tf.function(f_tf, jit_compile=True, autograph=False)
     with contextlib.ExitStack() as stack:
       if with_mesh:
         stack.enter_context(mesh)
       # Run the JAX native version, to check it works, and to fill caches.
       _ = func_to_convert(*args)
-      tf_hlo = f_tf.experimental_get_compiler_ir(*args)(stage="hlo")
+      exported = jax_export.serialize_native(
+          func_to_convert,
+          [core.ShapedArray(a.shape, a.dtype) for a in args],
+          lowering_platform='tpu',
+          strict_checks=True)
 
     if transform1 == "shard_map":
-      self.assertIn(" all-gather(f32[4,6]", tf_hlo)
+      self.assertIn("stablehlo.all_gather", str(exported.mlir_module))
     else:
-      self.assertIn(" reduce-window(", tf_hlo)
+      self.assertIn("stablehlo.reduce_window", str(exported.mlir_module))
+
+  def test_cross_platform_error(self):
+    f_tf = jax2tf.convert(jnp.sin, native_serialization=True,
+                          native_serialization_platforms=('tpu',))
+    x = np.float32(.5)
+    if jtu.device_under_test() == "tpu":
+      self.assertAllClose(jnp.sin(x), f_tf(x))
+    else:
+      # We can construct the tf.Graph
+      f_tf_fun = tf.function(f_tf, jit_compile=True, autograph=False)
+      graph_def = f_tf_fun.get_concrete_function(x).graph.as_graph_def()
+      self.assertIn("XlaCallModule", str(graph_def))
+      with self.assertRaisesRegex(tf.errors.NotFoundError,
+          "The current platform .* is not among the platforms required by the module"):
+        f_tf(x)
+
+  @jtu.ignore_warning(message="using native_serialization_platforms without native_serialization")
+  def test_native_parameters_for_non_native(self):
+    # We can use the native_serialization_platforms even for non-native
+    # serialization.
+    f_tf = jax2tf.convert(jnp.sin,
+                          native_serialization_platforms=('cpu',))
+    x = np.float32(.5)
+    # Run the TF code on CPU
+    tf_cpus = tf.config.list_logical_devices("CPU")
+    self.assertNotEmpty(tf_cpus)
+    with tf.device(tf_cpus[0]):
+      self.assertAllClose(jnp.sin(x), f_tf(x))
+
+    f_tf = jax2tf.convert(jnp.sin,
+                          native_serialization_strict_checks=False)
+    self.assertAllClose(jnp.sin(x), f_tf(x))
+
+  def test_native_serialization_grad(self):
+    # Check that the grad function uses the same native serialization parameters
+    # as the primal function.
+    f_tf = jax2tf.convert(jnp.sin, native_serialization=True,
+                          native_serialization_platforms=('tpu',))
+    x = np.arange(4, dtype=np.float32)
+    x_v = tf.Variable(x)
+
+    @tf.function(autograph=False)
+    def f_grad_tf(x_v):
+      with tf.GradientTape() as tape:
+        tape.watch(x_v)
+        res_tf = f_tf(x_v)
+        return tape.gradient(res_tf, x_v)
+
+    # Make sure that we have 2x XlaCallModule in the graph of the gradient
+    # function
+    f_grad_tf_fun = tf.function(f_grad_tf, autograph=False)
+    graph_def = f_grad_tf_fun.get_concrete_function(x).graph.as_graph_def()
+    logging.info("Found graph_def: %s", graph_def)
+    self.assertLen(re.findall(r'op:\s*"XlaCallModule"', str(graph_def)), 2)
+
+    if jtu.device_under_test() != "tpu":
+      with self.assertRaisesRegex(
+          tf.errors.NotFoundError,
+          r"The current platform .* is not among the platforms required by the module: \[TPU\]"):
+        f_grad_tf(x_v)
 
 
 def get_serialized_computation(
@@ -1503,7 +1594,7 @@ def get_serialized_computation(
     abstracted_axes: Optional[Tuple[Dict[int, str]]] = None,
     use_pjit: bool = False,
     in_shardings = None,
-    out_shardings = None) -> str:
+    out_shardings = None) -> Tuple[str, int]:
   if use_pjit:
     assert not abstracted_axes
     lowered = pjit.pjit(
@@ -1512,8 +1603,8 @@ def get_serialized_computation(
   else:
     lowered = jax.jit(f_jax, abstracted_axes=abstracted_axes).lower(*args)
   stablehlo_module_text = mlir.module_to_string(lowered._lowering.stablehlo())
-  logging.info(f'Serialized ir.Module = {stablehlo_module_text}')
-  return stablehlo_module_text
+  logging.info("Serialized ir.Module = %s", stablehlo_module_text)
+  return stablehlo_module_text, 3
 
 
 class XlaCallModuleTest(tf_test_util.JaxToTfTestCase):
@@ -1526,9 +1617,10 @@ class XlaCallModuleTest(tf_test_util.JaxToTfTestCase):
     x = np.ones((2, 3), dtype=np.float32)
 
     jax_res = f_jax(x)
+    module, version = get_serialized_computation(f_jax, x)
     res = tfxla.call_module([x],
-                            version=2,
-                            module=get_serialized_computation(f_jax, x),
+                            version=version,
+                            module=module,
                             Tout=[jax_res.dtype],
                             Sout=[jax_res.shape])
     self.assertAllClose(tf.nest.map_structure(lambda t: t.numpy(), res),
@@ -1544,9 +1636,10 @@ class XlaCallModuleTest(tf_test_util.JaxToTfTestCase):
     x = np.ones((2, 3), dtype=np.float32)
 
     jax_res = f_jax(count, x)
+    module, version = get_serialized_computation(f_jax, count, x)
     res = tfxla.call_module([count, x],
-                            version=2,
-                            module=get_serialized_computation(f_jax, count, x),
+                            version=version,
+                            module=module,
                             Tout=[jax_res.dtype],
                             Sout=[jax_res.shape])
     self.assertAllClose(tf.nest.map_structure(lambda t: t.numpy(), res),
@@ -1561,11 +1654,11 @@ class XlaCallModuleTest(tf_test_util.JaxToTfTestCase):
     x2 = np.ones((3, 4), dtype=np.float32)
 
     jax_res = f_jax(x1, x2)
-
+    module, version = get_serialized_computation(f_jax, x1, x2)
     def f_tf(x1_tf, x2_tf):
       return tfxla.call_module([x1_tf, x2_tf],
-                               version=2,
-                               module=get_serialized_computation(f_jax, x1, x2),
+                               version=version,
+                               module=module,
                                Tout=[jax_res[0].dtype, jax_res[1].dtype],
                                Sout=[jax_res[0].shape, jax_res[1].shape])
 
@@ -1573,32 +1666,6 @@ class XlaCallModuleTest(tf_test_util.JaxToTfTestCase):
     self.assertAllClose(tf.nest.map_structure(lambda t: t.numpy(), res),
                         jax_res)
 
-  @unittest.skip("TODO(necula): 'dynamic_iota' op can't be translated to XLA HLO")
-  def test_shape_poly_arange(self):
-    if not config.jax_dynamic_shapes:
-      raise unittest.SkipTest("jax_dynamic_shapes must be enabled")
-    def f_jax(x):  # x: f32[b]
-      return jnp.arange(x.shape[0]) + x
-
-    x1 = np.ones((5,), dtype=np.float32)
-    jax_res = f_jax(x1)
-
-    def f_tf(x1_tf):
-      return tfxla.call_module([x1_tf],
-                               version=2,
-                               module=get_serialized_computation(
-                                   f_jax, x1,
-                                   abstracted_axes=({
-                                       0: 'b'
-                                   },)),
-                               Tout=[jax_res.dtype],
-                               Sout=[jax_res.shape],
-                               dim_args_spec=('0.0',))
-
-    res = tf.function(f_tf, jit_compile=True, autograph=False)(x1)
-    self.assertAllClose(
-        tf.nest.map_structure(lambda t: t.numpy(), res), jax_res)
-
   @jtu.with_mesh([("x", 2)])
   def test_pjit_basic1D(self):
 
@@ -1614,7 +1681,7 @@ class XlaCallModuleTest(tf_test_util.JaxToTfTestCase):
         in_shardings=in_axis_resources,
         out_shardings=out_axis_resources,
     )(x, x)
-    module = get_serialized_computation(
+    module, version = get_serialized_computation(
         func_jax,
         x,
         x,
@@ -1624,7 +1691,7 @@ class XlaCallModuleTest(tf_test_util.JaxToTfTestCase):
 
     def f_tf(x_tf, y_tf):
       return tfxla.call_module([x_tf, y_tf],
-                               version=2,
+                               version=version,
                                module=module,
                                Tout=[x.dtype],
                                Sout=[x.shape])
