@@ -114,20 +114,15 @@ class JaxPrimitiveTest(tf_test_util.JaxToTfTestCase):
     func_jax = harness.dyn_fun
     args = harness.dyn_args_maker(self.rng())
     enable_xla = harness.params.get("enable_xla", True)
-    if config.jax2tf_default_experimental_native_lowering and not enable_xla:
-      raise unittest.SkipTest("experimental_native_lowering not supported with enable_xla=False")
+    if config.jax2tf_default_native_serialization and not enable_xla:
+      raise unittest.SkipTest("native_serialization not supported with enable_xla=False")
 
     if ("eigh" == harness.group_name and
         np.complex64 == harness.dtype and
         device == "tpu"):
       raise unittest.SkipTest("b/264716764: error on tf.cast from c64 to f32")
 
-    if (not config.jax_array and
-        device == "cpu" and
-        "top_k_sort_inf_nan_inshape=float32[5]_k=5" in harness.fullname):
-      raise unittest.SkipTest("Unexplained failure, but in old no_jax_array")
-
-    if (config.jax2tf_default_experimental_native_lowering and
+    if (config.jax2tf_default_native_serialization and
         device == "gpu" and
         "lu" in harness.fullname):
       raise unittest.SkipTest("b/269388847: lu failures on GPU")
@@ -135,53 +130,30 @@ class JaxPrimitiveTest(tf_test_util.JaxToTfTestCase):
     def skipCustomCallTest(target: str):
       raise unittest.SkipTest(
           f"TODO(b/272239584): custom call target not guaranteed stable: {target}")
-    if config.jax2tf_default_experimental_native_lowering:
+    if config.jax2tf_default_native_serialization:
       if device == "cpu":
         if "cholesky_shape" in harness.fullname:
           skipCustomCallTest("lapack_spotrf, lapack_dpotrf, lapack_zpotrf, lapack_cpotrf")
         if "eig_shape" in harness.fullname:
           skipCustomCallTest("lapack_cgeev, lapack_sgeev, lapack_dgeev, lapack_zgeev")
-        if "eigh_shape" in harness.fullname:
-          skipCustomCallTest("lapack_cheevd, lapack_ssyevd, lapack_zheevd")
         if "lu_shape" in harness.fullname:
           skipCustomCallTest("lapack_zgetrf, lapack_sgetrf")
         if "svd_shape" in harness.fullname:
           skipCustomCallTest("lapack_sgesdd, lapack_zgesdd, lapack_cgesdd")
-        if "qr_" in harness.fullname:
-          skipCustomCallTest("lapack_dgeqrf, lapack_cgeqrf, lapack_zgeqrf")
         if "triangular_solve_" in harness.fullname:
           skipCustomCallTest("blas_ctrsm, blas_dtrsm, blas_ztrsm, blas_strsm")
-        if "fft_" in harness.fullname:
-          skipCustomCallTest("ducc_fft")
         if "custom_linear_solve" in harness.fullname:
           skipCustomCallTest("lapack_sgetrf, lapack_dgetrf")
 
       elif device == "tpu":
-        if "qr_" in harness.fullname:
-          skipCustomCallTest("Qr")
-        if "svd_shape" in harness.fullname:
-          skipCustomCallTest("Qr")
-        if "lu_shape" in harness.fullname:
-          skipCustomCallTest("LuDecomposition")
-        if "custom_linear_solve_" in harness.fullname:
-          skipCustomCallTest("LuDecomposition")
-        if "eigh_shape" in harness.fullname:
-          skipCustomCallTest("Eigh")
+        if "approx_top_k_large=True" in harness.fullname:
+          skipCustomCallTest("PartialReduce")  # ApproxTopK
 
       elif device == "gpu":
-        if "eigh_shape" in harness.fullname:
-          skipCustomCallTest("cusolver_syevj")
-        if ("qr_" in harness.fullname or
-            "custom_linear_solve_" in harness.fullname):
+        if "custom_linear_solve_" in harness.fullname:
           skipCustomCallTest("cusolver_geqrf, cublas_geqrf_batched")
         if "svd_shape" in harness.fullname:
           skipCustomCallTest("cusolver_gesvdj")
-        if ("random_split_" in harness.fullname or
-            "random_gamma_" in harness.fullname or
-            "random_uniform_" in harness.fullname or
-            "random_categorical_" in harness.fullname or
-            "random_randint" in harness.fullname):
-          skipCustomCallTest("cu_threefry2x32")
         if "tridiagonal_solve_shape" in harness.fullname:
           skipCustomCallTest("cusparse_gtsv2_f32, cusparse_gtsv2_f64")
 
@@ -192,10 +164,10 @@ class JaxPrimitiveTest(tf_test_util.JaxToTfTestCase):
                                enable_xla=enable_xla)
     except Exception as e:
       # TODO(b/264596006): custom calls are not registered properly with TF in OSS
-      if (config.jax2tf_default_experimental_native_lowering and
+      if (config.jax2tf_default_native_serialization and
           "does not work with custom calls" in str(e)):
         logging.warning("Supressing error %s", e)
-        raise unittest.SkipTest("b/264596006: custom calls in native lowering fail in TF")
+        raise unittest.SkipTest("b/264596006: custom calls in native serialization fail in TF")
       else:
         raise e
 
@@ -346,8 +318,8 @@ class JaxPrimitiveTest(tf_test_util.JaxToTfTestCase):
     y = np.int32(3)
     self.ConvertAndCompare(jnp.floor_divide, x, y)
     expected = jnp.floor_divide(x, y)
-    if not config.jax2tf_default_experimental_native_lowering:
-      # With native lowering TF1 seems to want to run the converted code
+    if not config.jax2tf_default_native_serialization:
+      # With native serialization TF1 seems to want to run the converted code
       # on the CPU even when the default backend is the TPU.
       # Try it with TF 1 as well (#5831)
       with tf.compat.v1.Session() as sess:
