@@ -24,14 +24,12 @@ from jax.tree_util import tree_flatten, tree_map, tree_unflatten
 from jax._src import core
 from jax._src import dispatch
 from jax._src import array
-from jax._src import sharding
-from jax.tree_util import PyTreeDef
+from jax._src import sharding_impls
 from jax._src.interpreters import pxla
 from jax.interpreters import xla
 from jax._src import pjit as pjit_lib
-from jax.experimental.pjit import pjit, FROM_GDA
+from jax.experimental.pjit import pjit
 from jax.sharding import PartitionSpec as P
-from jax._src.global_device_array import GlobalDeviceArray
 from jax._src import distributed
 from jax._src import config as config_internal
 import numpy as np
@@ -39,12 +37,11 @@ import numpy as np
 
 # This needs to be top-level for the jax compilation cache.
 @functools.partial(jax.pmap, axis_name='hosts')
-def _psum(x: PyTreeDef) -> PyTreeDef:
+def _psum(x: Any) -> Any:
   return jax.lax.psum(x, 'hosts')
 
 
-def broadcast_one_to_all(in_tree: PyTreeDef,
-                         is_source: Optional[bool] = None) -> PyTreeDef:
+def broadcast_one_to_all(in_tree: Any, is_source: Optional[bool] = None) -> Any:
   """Broadcast data from a source host (host 0 by default) to all other hosts.
 
   Args:
@@ -62,9 +59,6 @@ def broadcast_one_to_all(in_tree: PyTreeDef,
     is_source = jax.process_index() == 0
 
   def pre_pmap(x):
-    if isinstance(x, GlobalDeviceArray):
-      raise ValueError('GDAs cannot be broadcasted from source host to other '
-                       'hosts.')
     if is_source:
       return np.concatenate([
           x[None, ...],
@@ -96,8 +90,9 @@ def _identity_fn(x):
 
 def _handle_array_process_allgather(inp, tiled):
   if isinstance(inp, array.ArrayImpl) and not inp.is_fully_addressable:
-    reps = sharding.GSPMDSharding(inp.sharding._device_assignment,
-                                       sharding._get_replicated_op_sharding())
+    reps = sharding_impls.GSPMDSharding(
+        inp.sharding._device_assignment,
+        sharding_impls.get_replicated_op_sharding())
     out = pjit(_identity_fn, out_shardings=reps)(inp)
   else:
     # All inputs here will be fully addressable.
@@ -115,8 +110,8 @@ def _handle_array_process_allgather(inp, tiled):
       host_np_arr = np.expand_dims(host_np_arr, axis=0)
 
     aval = core.ShapedArray(host_np_arr.shape, host_np_arr.dtype)
-    global_aval = global_mesh._local_to_global(
-        pxla.get_array_mapping(pspec), aval)
+    global_aval = pxla.mesh_local_to_global(
+        global_mesh, pxla.get_array_mapping(pspec), aval)
 
     bufs = [jax.device_put(host_np_arr, d) for d in jax.local_devices()]
     global_arr = array.make_array_from_single_device_arrays(
@@ -127,7 +122,7 @@ def _handle_array_process_allgather(inp, tiled):
   return np.asarray(out.addressable_data(0))
 
 
-def process_allgather(in_tree: PyTreeDef, tiled: bool = False) -> PyTreeDef:
+def process_allgather(in_tree: Any, tiled: bool = False) -> Any:
   """Gather data from across processes.
 
   Args:
@@ -149,35 +144,8 @@ def process_allgather(in_tree: PyTreeDef, tiled: bool = False) -> PyTreeDef:
   """
 
   def _pjit(inp):
-    if jax.config.jax_array:
-      return _handle_array_process_allgather(inp, tiled)
-    else:
-      if isinstance(inp, GlobalDeviceArray):
-        if inp.is_fully_replicated:
-          return np.asarray(inp.addressable_data(0))
-        global_mesh = inp.mesh
-        in_axis_resources = FROM_GDA
-      else:
-        # DA/SDA/np.array will be sharded based on global_mesh.local_mesh.
-        # Shape of local_mesh will always be (1, local_device_count())
-        devices = np.array(jax.devices()).reshape(jax.process_count(),
-                                                  jax.local_device_count())
-        global_mesh = jax.sharding.Mesh(devices, ('processes', 'local_devices'))
-        in_axis_resources = P('processes')
-        if inp.ndim == 0 or not tiled:
-          inp = np.expand_dims(inp, axis=0)
-
-      with global_mesh:
-        out = pjit(
-            _identity_fn, in_shardings=in_axis_resources, out_shardings=None
-        )(inp)
-      return np.asarray(out.addressable_data(0))
-
-  if jax.config.jax_array:
-    return jax.tree_map(_pjit, in_tree)  # array route
-  else:
-    with config_internal.parallel_functions_output_gda(True):
-      return jax.tree_map(_pjit, in_tree)  # gda route
+    return _handle_array_process_allgather(inp, tiled)
+  return jax.tree_map(_pjit, in_tree)
 
 
 def assert_equal(in_tree, fail_message: str = ''):
@@ -243,18 +211,13 @@ def _flatten_pspecs(name, in_tree, pspecs_thunk):
 
 @functools.lru_cache()
 def _local_to_global_aval(local_aval, mesh, pspec):
-  return mesh._local_to_global(pxla.get_array_mapping(pspec), local_aval)
+  return pxla.mesh_local_to_global(mesh, pxla.get_array_mapping(pspec),
+                                   local_aval)
 
 @functools.lru_cache()
 def _global_to_local_aval(global_aval, mesh, pspec):
-  return mesh._global_to_local(
-      pxla.get_array_mapping(pspec), global_aval)
-
-def _device_put(x, device):
-  try:
-    return dispatch.device_put_handlers[type(x)](x, device)
-  except KeyError as err:
-    raise TypeError(f"No device_put handler for type: {type(x)}") from err
+  return pxla.mesh_global_to_local(mesh, pxla.get_array_mapping(pspec),
+                                   global_aval)
 
 
 def host_local_array_to_global_array(local_inputs: Any,
@@ -289,17 +252,7 @@ def host_local_array_to_global_array(local_inputs: Any,
     local_inputs: A Pytree of host local values.
     global_mesh: A ``jax.sharding.Mesh`` object.
     pspecs: A Pytree of ``jax.sharding.PartitionSpec``s.
-
-  Raises:
-    RuntimeError: If `jax.config.jax_array` not previously enabled.
   """
-  if not jax.config.jax_array:
-    raise RuntimeError(
-        "Please enable `jax_array` to use `host_local_array_to_global_array`. "
-        "You can use jax.config.update('jax_array', True) or set the "
-        "environment variable  JAX_ARRAY=1 , or set the `jax_array` boolean "
-        "flag to something true-like.")
-
   def _convert(arr, pspec):
     # If the Array is not fully addressable i.e. not host local, return it.
     if isinstance(arr, array.ArrayImpl) and not arr.is_fully_addressable:
@@ -314,24 +267,20 @@ def host_local_array_to_global_array(local_inputs: Any,
     # matches the `local_sharding`, then there's no need to reshard and create
     # copies.
     if (isinstance(arr, array.ArrayImpl) and
-        arr.sharding._device_assignment == local_sharding._device_assignment and
-        pxla.are_op_shardings_equal(
-            arr.sharding._to_xla_op_sharding(arr.ndim),
-            local_sharding._to_xla_op_sharding(arr.ndim))):
-      arrays = arr._arrays
+        arr.sharding.is_equivalent_to(local_sharding, arr.ndim)):
+      arrays = [x.data for x in arr.addressable_shards]
     else:
       arr = xla.canonicalize_dtype(arr)
-      arrays = list(it.chain.from_iterable(
-          _device_put(arr[index], d)
-          for d, index in local_sharding.devices_indices_map(arr.shape).items()
-      ))
+      arrays = list(
+          arr[index]
+          for d, index in local_sharding.devices_indices_map(arr.shape).items())
 
     global_aval = _local_to_global_aval(
         core.ShapedArray(arr.shape, arrays[0].dtype), global_mesh, pspec)
 
-    return array.ArrayImpl(
+    return pxla.batched_device_put(
         global_aval, jax.sharding.NamedSharding(global_mesh, pspec),
-        arrays, committed=True, _skip_checks=True)
+        arrays, list(global_mesh.local_mesh.devices.flat))
 
   flattened_inps, in_tree = tree_flatten(local_inputs)
   in_pspecs = _flatten_pspecs('input pspecs', in_tree,
@@ -373,17 +322,7 @@ def global_array_to_host_local_array(global_inputs: Any,
     global_inputs: A Pytree of global `jax.Array`s.
     global_mesh: A ``jax.sharding.Mesh`` object.
     pspecs: A Pytree of ``jax.sharding.PartitionSpec``s.
-
-  Raises:
-    RuntimeError: If `jax.config.jax_array` not previously enabled.
   """
-  if not jax.config.jax_array:
-    raise RuntimeError(
-        "Please enable `jax_array` to use `global_array_to_host_local_array`. "
-        "You can use jax.config.update('jax_array', True) or set the "
-        "environment variable  JAX_ARRAY=1 , or set the `jax_array` boolean "
-        "flag to something true-like.")
-
   def _convert(arr, pspec):
     # If the Array is already fully addressable i.e. host local, return it.
     if isinstance(arr, array.ArrayImpl) and arr.is_fully_addressable:
