@@ -16,7 +16,6 @@
 
 from jax._src.pjit import (
   AUTO as AUTO,
-  FROM_GDA as FROM_GDA,
   ParsedPartitionSpec as ParsedPartitionSpec,
   get_array_mapping as get_array_mapping,
   hashable_pytree as hashable_pytree,
@@ -30,7 +29,7 @@ from jax._src.pjit import (_UNSPECIFIED, _prepare_axis_resources,
                            _get_op_sharding_from_executable,
                            _get_pspec_from_executable, _pjit_lower_cached,
                            _pjit_lower, _get_op_sharding,
-                           _calc_is_global_sequence, _pjit_jaxpr,
+                           _pjit_jaxpr,
                            _create_mesh_pspec_sharding_from_parsed_pspec,
                            _process_in_axis_resources)
 
@@ -49,17 +48,29 @@ if typing.TYPE_CHECKING:
 del typing
 
 _deprecations = {
-  # Added Feb 13, 2023:
-  "NamedSharding": (
-    ("jax.experimental.pjit.NamedSharding is deprecated. Use "
-     "jax.sharding.NamedSharding."),
-    _deprecated_NamedSharding,
-  ),
-  "PartitionSpec": (
-    ("jax.experimental.pjit.PartitionSpec is deprecated. Use "
-     "jax.sharding.PartitionSpec."),
-     _deprecated_PartitionSpec,
-  ),
+    # Added Feb 13, 2023:
+    "NamedSharding": (
+        (
+            "jax.experimental.pjit.NamedSharding is deprecated. Use "
+            "jax.sharding.NamedSharding."
+        ),
+        _deprecated_NamedSharding,
+    ),
+    "PartitionSpec": (
+        (
+            "jax.experimental.pjit.PartitionSpec is deprecated. Use "
+            "jax.sharding.PartitionSpec."
+        ),
+        _deprecated_PartitionSpec,
+    ),
+    "FROM_GDA": (
+        (
+            "jax.experimental.pjit.FROM_GDA has been removed. Please pass in"
+            " sharded jax.Arrays as input and remove the in_shardings argument"
+            " to pjit since pjit will infer the shardings from jax.Array."
+        ),
+        None,
+    ),
 }
 
 from jax._src.deprecations import deprecation_getattr as _deprecation_getattr
