@@ -39,19 +39,20 @@ from jax._src import util
 from jax._src.core import Tracer
 from jax._src.lax import (lax, parallel as lax_parallel, slicing,
                           windowed_reductions, fft, linalg)
-from jax._src.util import (HashableFunction, unzip2, as_hashable_function,
-                           memoize, partition_list, merge_lists)
+from jax._src.util import (HashableFunction, HashablePartial, unzip2,
+                           as_hashable_function, memoize, partition_list,
+                           merge_lists)
 from jax.api_util import flatten_fun_nokwargs, shaped_abstractify
-from jax.interpreters import batching
+from jax._src.interpreters import batching
 from jax._src.interpreters import mlir
-from jax.interpreters import partial_eval as pe
-from jax.interpreters import xla
+from jax._src.interpreters import partial_eval as pe
+from jax._src.interpreters import xla
 from jax._src.interpreters import pxla
 from jax.interpreters import ad
 from jax.tree_util import (tree_map, tree_flatten, tree_unflatten,
                            tree_structure, tree_leaves, keystr)
 from jax._src.tree_util import (broadcast_prefix, prefix_errors, PyTreeDef,
-                                _generate_key_paths, KeyPath)
+                                generate_key_paths, KeyPath)
 
 P = PartitionSpec
 
@@ -132,7 +133,7 @@ def _check_specs(error_type: SpecErrorType, specs: Any) -> None:
   if all(isinstance(p, PartitionSpec) for p in tree_leaves(specs)): return
   prefix = 'in' if error_type == SpecErrorType.input else 'out'
   msgs = [f"  {prefix}_specs{keystr(key)} is {x} of type {type(x).__name__}, "
-          for key, x in _generate_key_paths(specs) if not isinstance(x, P)]
+          for key, x in generate_key_paths(specs) if not isinstance(x, P)]
   raise TypeError(
       f"shard_map {prefix}_specs argument must be a pytree of "
       f"`jax.sharding.PartitionSpec` instances, but:\n\n"
@@ -257,7 +258,7 @@ def _rep_error(f: Callable, mesh: Mesh, tree: PyTreeDef, specs: Specs,
          + '\n\n'.join(msgs) + '\n\n' +
          "Check if these output values are meant to be replicated over those "
          "mesh axes. If not, consider revising the corresponding out_specs "
-         "entries. If so, consider ddisabling the check by passing the "
+         "entries. If so, consider disabling the check by passing the "
          "check_rep=False argument to shard_map.")
   return msg
 
@@ -275,8 +276,8 @@ T = TypeVar('T')
 def _iter_paths(tree: PyTreeDef, specs: Specs, fails: List[Union[T, NoFail]]
                 ) -> List[Tuple[Tuple[KeyPath, P], Tuple[KeyPath, T]]]:
   failures = tree_unflatten(tree, fails)
-  failures_aug = _generate_key_paths(failures)
-  specs_ = tree_unflatten(tree_structure(specs), _generate_key_paths(specs))
+  failures_aug = generate_key_paths(failures)
+  specs_ = tree_unflatten(tree_structure(specs), generate_key_paths(specs))
   leaf = lambda x: type(x) is tuple and len(x) == 2 and type(x[1]) is P
   specs_aug = broadcast_prefix(specs_, failures, is_leaf=leaf)
   return [((spec_key, spec), (fail_key, fail_data))
@@ -401,7 +402,7 @@ def _unshard_aval(mesh: Mesh, names: AxisNames, aval: core.AbstractValue
 
 # Type-checking
 
-def _shard_map_typecheck(*in_atoms, jaxpr, mesh, in_names, out_names,
+def _shard_map_typecheck(_, *in_atoms, jaxpr, mesh, in_names, out_names,
                          check_rep):
   for v, x, in_name in zip(jaxpr.invars, in_atoms, in_names):
     if not core.typecompat(v.aval, _shard_aval(mesh, in_name, x.aval)):
@@ -580,25 +581,8 @@ class ShardMapTrace(core.Trace):
     return ShardMapTracer(self, out_rep, out_vals)
 
   def process_call(self, call_primitive, fun, tracers, params):
-    if call_primitive is not xla.xla_call_p: raise NotImplementedError
-    fun, jaxpr = _grab_jaxpr_shadily(fun)  # TODO remove with initial-style jit
-    bind = partial(call_primitive.bind, fun)  # TODO caching (compat w/ jaxpr())
-    fake_primitive = pxla.FakePrimitive(multiple_results=True, bind=bind)
-    _rep_rules[fake_primitive] = lambda *_, **__: set()  # pytype: disable=container-type-mismatch
-    out_tracers_ = self.process_primitive(fake_primitive, tracers, params)
-    out_vals = [t.val for t in out_tracers_]
-    if self.check:
-      out_rep = _output_rep(self.mesh, jaxpr(), [t.rep for t in tracers])
-    else:
-      out_rep = [set()] * len(out_vals)
-    return map(partial(ShardMapTracer, self), out_rep, out_vals)
+    raise NotImplementedError
 
-@lu.transformation_with_aux
-def _grab_jaxpr_shadily(*args):
-  out = yield args, {}
-  main = core.thread_local_state.trace_state.trace_stack.dynamic  # forgive me
-  jaxpr, _ = main.jaxpr_stack[-1].to_jaxpr(out)
-  yield out, jaxpr
 
 class ShardMapTracer(core.Tracer):
   rep: Set[AxisName]
@@ -710,10 +694,6 @@ def _axis_index_rule(mesh, *, axis_name):
 def _pjit_rule(mesh, *in_rep, jaxpr, **kwargs):
   return _output_rep(mesh, jaxpr.jaxpr, in_rep)
 
-@register_rule(xla.xla_call_p)
-def _jit_rule(mesh, *in_rep, jaxpr, **kwargs):
-  return _output_rep(mesh, jaxpr, in_rep)
-
 @register_rule(debugging.debug_callback_p)
 def _debug_callback_rule(mesh, *in_rep, **_):
   return []
@@ -978,26 +958,9 @@ def _pe_custom_res(params_known, aval):
   mesh = params_known['mesh']
   return _unshard_aval(mesh, {0: (*mesh.axis_names,)}, aval)
 
+def _pe_custom_ctx(params):
+  return core.extend_axis_env_nd(params['mesh'].shape.items())
+
 pe.partial_eval_jaxpr_custom_rules[shard_map_p] = \
     partial(pe.call_partial_eval_custom_rule, 'jaxpr', _pe_custom_params,
-            res_aval=_pe_custom_res)
-
-# Misc
-
-# TODO(mattjj): move this to _src/util.py
-class HashablePartial:
-  def __init__(self, f, *args, **kwargs):
-    self.f = f
-    self.args = args
-    self.kwargs = kwargs
-
-  def __eq__(self, other):
-    return (type(other) is HashablePartial and
-            self.f.__code__ == other.f.__code__ and
-            self.args == other.args and self.kwargs == other.kwargs)
-
-  def __hash__(self):
-    return hash((self.f.__code__, self.args, tuple(self.kwargs.items())))
-
-  def __call__(self, *args, **kwargs):
-    return self.f(*self.args, *args, **self.kwargs, **kwargs)
+            res_aval=_pe_custom_res, ctx=_pe_custom_ctx)
