@@ -34,7 +34,7 @@ from jax._src.interpreters import ad
 from jax._src.lax.lax import _const
 from jax._src.lib.mlir.dialects import hlo
 from jax._src.lib import gpu_sparse
-from jax._src.numpy.util import _promote_dtypes
+from jax._src.numpy.util import promote_dtypes
 from jax._src.typing import Array, ArrayLike, DTypeLike
 import jax.numpy as jnp
 
@@ -156,7 +156,7 @@ class COO(JAXSparse):
     if isinstance(other, JAXSparse):
       raise NotImplementedError("matmul between two sparse objects.")
     other = jnp.asarray(other)
-    data, other = _promote_dtypes(self.data, other)
+    data, other = promote_dtypes(self.data, other)
     self_promoted = COO((data, self.row, self.col), **self._info._asdict())
     if other.ndim == 1:
       return coo_matvec(self_promoted, other)
@@ -221,7 +221,7 @@ def _coo_todense_gpu_lowering(coo_todense_hlo, ctx, data, row, col, *, spinfo):
     shape = spinfo.shape[::-1]
   else:
     warnings.warn("coo_todense GPU lowering requires matrices with sorted rows or sorted cols. "
-                  "To sort the rows in your matrix, use e.g. mat = mat._sort_rows(). Falling "
+                  "To sort the rows in your matrix, use e.g. mat = mat._sort_indices(). Falling "
                   "back to the default implementation.", CuSparseEfficiencyWarning)
     return _coo_todense_lowering(ctx, data, row, col, spinfo=spinfo)
 
@@ -460,7 +460,7 @@ def _coo_matvec_gpu_lowering(coo_matvec_hlo, ctx, data, row, col, v, *, spinfo,
     shape = spinfo.shape[::-1]
   else:
     warnings.warn("coo_matvec GPU lowering requires matrices with sorted rows or sorted cols. "
-                  "To sort the rows in your matrix, use e.g. mat = mat._sort_rows(). Falling "
+                  "To sort the rows in your matrix, use e.g. mat = mat._sort_indices(). Falling "
                   "back to the default implementation.", CuSparseEfficiencyWarning)
     return _coo_matvec_lowering(ctx, data, row, col, v, spinfo=spinfo,
                                 transpose=transpose)
@@ -583,7 +583,7 @@ def _coo_matmat_gpu_lowering(coo_matmat_hlo, ctx, data, row, col, B, *, spinfo,
     shape = spinfo.shape[::-1]
   else:
     warnings.warn("coo_matmat GPU lowering requires matrices with sorted rows or sorted cols. "
-                  "To sort the rows in your matrix, use e.g. mat = mat._sort_rows(). Falling "
+                  "To sort the rows in your matrix, use e.g. mat = mat._sort_indices(). Falling "
                   "back to the default implementation.", CuSparseEfficiencyWarning)
     return _coo_matmat_lowering(ctx, data, row, col, B, spinfo=spinfo,
                                 transpose=transpose)
