@@ -20,19 +20,21 @@ import functools
 import jax
 import jax.numpy as jnp
 
-from jax import core
 from jax.interpreters import mlir
 from jax.interpreters import xla
 
+from jax._src import core
 from jax._src.lib import gpu_solver
 
 import numpy as np
+from scipy.sparse import csr_matrix, linalg
+
 
 def lobpcg_standard(
-    A: Union[jnp.ndarray, Callable[[jnp.ndarray], jnp.ndarray]],
-    X: jnp.ndarray,
+    A: Union[jax.Array, Callable[[jax.Array], jax.Array]],
+    X: jax.Array,
     m: int = 100,
-    tol: Union[jnp.ndarray, float, None] = None):
+    tol: Union[jax.Array, float, None] = None):
   """Compute the top-k standard eigenvalues using the LOBPCG routine.
 
   LOBPCG [1] stands for Locally Optimal Block Preconditioned Conjugate Gradient.
@@ -93,16 +95,16 @@ def lobpcg_standard(
                  large (only `k * 5 < n` supported), or `k == 0`.
   """
   # Jit-compile once per matrix shape if possible.
-  if isinstance(A, (jnp.ndarray, np.ndarray)):
+  if isinstance(A, (jax.Array, np.ndarray)):
     return _lobpcg_standard_matrix(A, X, m, tol, debug=False)
   return _lobpcg_standard_callable(A, X, m, tol, debug=False)
 
 @functools.partial(jax.jit, static_argnames=['m', 'debug'])
 def _lobpcg_standard_matrix(
-    A: jnp.ndarray,
-    X: jnp.ndarray,
+    A: jax.Array,
+    X: jax.Array,
     m: int,
-    tol: Union[jnp.ndarray, float, None],
+    tol: Union[jax.Array, float, None],
     debug: bool = False):
   """Computes lobpcg_standard(), possibly with debug diagnostics."""
   return _lobpcg_standard_callable(
@@ -110,10 +112,10 @@ def _lobpcg_standard_matrix(
 
 @functools.partial(jax.jit, static_argnames=['A', 'm', 'debug'])
 def _lobpcg_standard_callable(
-    A: Callable[[jnp.ndarray], jnp.ndarray],
-    X: jnp.ndarray,
+    A: Callable[[jax.Array], jax.Array],
+    X: jax.Array,
     m: int,
-    tol: Union[jnp.ndarray, float, None],
+    tol: Union[jax.Array, float, None],
     debug: bool = False):
   """Supports generic lobpcg_standard() callable interface."""
 
@@ -510,24 +512,48 @@ def _extend_basis(X, m):
 
 
 # Sparse direct solve via QR factorization
+def _spsolve_abstract_eval(data, indices, indptr, b, *, tol, reorder):
+  if data.dtype != b.dtype:
+    raise ValueError(f"data types do not match: {data.dtype=} {b.dtype=}")
+  if not (jnp.issubdtype(indices.dtype, jnp.integer) and jnp.issubdtype(indptr.dtype, jnp.integer)):
+    raise ValueError(f"index arrays must be integer typed; got {indices.dtype=} {indptr.dtype=}")
+  if not data.ndim == indices.ndim == indptr.ndim == b.ndim == 1:
+    raise ValueError("Arrays must be one-dimensional. "
+                     f"Got {data.shape=} {indices.shape=} {indptr.shape=} {b.shape=}")
+  if indptr.size != b.size + 1 or  data.shape != indices.shape:
+    raise ValueError(f"Invalid CSR buffer sizes: {data.shape=} {indices.shape=} {indptr.shape=}")
+  if reorder not in [0, 1, 2, 3]:
+    raise ValueError(f"{reorder=} not valid, must be one of [1, 2, 3, 4]")
+  tol = float(tol)
+  return b
 
 
-def _spsolve_abstract_eval(data, indices, indptr, b, tol, reorder):
-  del data, indices, indptr, tol, reorder
-  return core.raise_to_shaped(b)
-
-
-def _spsolve_gpu_lowering(ctx, data, indices, indptr, b, tol, reorder):
+def _spsolve_gpu_lowering(ctx, data, indices, indptr, b, *, tol, reorder):
   data_aval, _, _, _, = ctx.avals_in
-
   return gpu_solver.cuda_csrlsvqr(data_aval.dtype, data, indices,
                                   indptr, b, tol, reorder)
 
 
+def _spsolve_cpu_lowering(ctx, data, indices, indptr, b, tol, reorder):
+  del tol, reorder
+  args = [data, indices, indptr, b]
+
+  def _callback(data, indices, indptr, b, **kwargs):
+    A = csr_matrix((data, indices, indptr), shape=(b.size, b.size))
+    return (linalg.spsolve(A, b).astype(b.dtype),)
+
+  result, _, keepalive = mlir.emit_python_callback(
+      ctx, _callback, None, args, ctx.avals_in, ctx.avals_out,
+      has_side_effect=False)
+  ctx.module_context.add_keepalive(keepalive)
+  return result
+
+
 spsolve_p = core.Primitive('spsolve')
 spsolve_p.def_impl(functools.partial(xla.apply_primitive, spsolve_p))
 spsolve_p.def_abstract_eval(_spsolve_abstract_eval)
 mlir.register_lowering(spsolve_p, _spsolve_gpu_lowering, platform='cuda')
+mlir.register_lowering(spsolve_p, _spsolve_cpu_lowering, platform='cpu')
 
 
 def spsolve(data, indices, indptr, b, tol=1e-6, reorder=1):
