@@ -62,9 +62,9 @@ from jax.experimental.sparse.bcoo import bcoo_multiply_dense, bcoo_multiply_spar
 import jax.numpy as jnp
 from jax._src.api_util import flatten_fun_nokwargs
 from jax._src.lib import pytree
-from jax.interpreters import partial_eval as pe
-from jax.interpreters import xla
-from jax.interpreters import pxla
+from jax._src.interpreters import partial_eval as pe
+from jax._src.interpreters import xla
+from jax._src.interpreters import pxla
 from jax.tree_util import tree_flatten, tree_map, tree_unflatten
 from jax.util import safe_map, safe_zip, split_list
 from jax._src.config import config
@@ -421,14 +421,7 @@ def eval_sparse(
         _raise_unimplemented_primitive(prim)
       out = sparse_rules_bcoo[prim](spenv, *invals, **eqn.params)
     else:
-      if prim is xla.xla_call_p:
-        # TODO(vanderplas,frostig): workaround for binding call primitives
-        # within a jaxpr interpreter
-        params = eqn.params.copy()
-        fun = lu.wrap_init(core.jaxpr_as_fun(pe.ClosedJaxpr(params.pop('call_jaxpr'), ())))
-        out_bufs = prim.bind(fun, *(spenv.data(val) for val in invals), **params)
-      else:
-        out_bufs = prim.bind(*(spenv.data(val) for val in invals), **eqn.params)
+      out_bufs = prim.bind(*(spenv.data(val) for val in invals), **eqn.params)
       out_bufs = out_bufs if prim.multiple_results else [out_bufs]
       out = []
       for buf, outvar in safe_zip(out_bufs, eqn.outvars):
@@ -671,6 +664,22 @@ def _mul_sparse(spenv, *spvalues):
 
 sparse_rules_bcoo[lax.mul_p] = _mul_sparse
 
+def _div_sparse(spenv, *spvalues):
+  X, Y = spvalues
+  if Y.is_sparse():
+    raise NotImplementedError(
+      "Division by a sparse array is not implemented because it "
+      "would result in dense output. If this is your intent, use "
+      "sparse.todense() to convert your arguments to a dense array.")
+  X_promoted = spvalues_to_arrays(spenv, X)
+  out_data = bcoo_multiply_dense(X_promoted, 1. / spenv.data(Y))
+  out_spvalue = spenv.sparse(X.shape, out_data, indices_ref=X.indices_ref,
+                              indices_sorted=X.indices_sorted,
+                              unique_indices=X.unique_indices)
+  return (out_spvalue,)
+
+sparse_rules_bcoo[lax.div_p] = _div_sparse
+
 def _reduce_sum_sparse(spenv, *spvalues, axes):
   X, = spvalues
   X_promoted = spvalues_to_arrays(spenv, X)
@@ -743,34 +752,15 @@ def _while_sparse(spenv, *spvalues, cond_jaxpr, cond_nconsts, body_jaxpr, body_n
 
 sparse_rules_bcoo[lax.while_p] = _while_sparse
 
-def _xla_call_sparse(spenv, *spvalues, call_jaxpr, donated_invars, **params):
-  if any(donated_invars):
-    raise NotImplementedError("sparse xla_call with donated_invars")
-  sp_call_jaxpr, out_tree = _sparsify_jaxpr(spenv, pe.ClosedJaxpr(call_jaxpr, ()), *spvalues)
-  fun = lu.wrap_init(core.jaxpr_as_fun(sp_call_jaxpr))
-  args_flat, _ = tree_flatten(spvalues_to_arrays(spenv, spvalues))
-  donated_invars = tuple(False for arg in args_flat)
-  out_flat = xla.xla_call_p.bind(fun, *args_flat, donated_invars=donated_invars, **params)
-  return arrays_to_spvalues(spenv, tree_unflatten(out_tree, out_flat))
-
-sparse_rules_bcoo[xla.xla_call_p] = _xla_call_sparse
-
 
 def _pjit_sparse(spenv, *spvalues, jaxpr, in_shardings, out_shardings,
-                 resource_env, donated_invars, name, in_positional_semantics,
-                 out_positional_semantics, keep_unused, inline):
-  if not config.jax_array:
-    raise ValueError('sparse pjit is only supported with jax.Array.')
+                 resource_env, donated_invars, name, keep_unused, inline):
   if any(donated_invars):
     raise NotImplementedError("sparse xla_call with donated_invars")
 
   sp_call_jaxpr, out_tree = _sparsify_jaxpr(spenv, jaxpr, *spvalues)
   args_flat, _ = tree_flatten(spvalues_to_arrays(spenv, spvalues))
   donated_invars = tuple(False for arg in args_flat)
-  in_positional_semantics = tuple(pxla._PositionalSemantics.GLOBAL
-                                  for _ in args_flat)
-  out_positional_semantics = tuple(pxla._PositionalSemantics.GLOBAL
-                                   for _ in sp_call_jaxpr.out_avals)
 
   # TODO(yashkatariya, vanderplas): Flatten twice and set the correct sharding
   # for data and indices.
@@ -787,8 +777,6 @@ def _pjit_sparse(spenv, *spvalues, jaxpr, in_shardings, out_shardings,
       resource_env=resource_env,
       donated_invars=donated_invars,
       name=name,
-      in_positional_semantics=in_positional_semantics,
-      out_positional_semantics=out_positional_semantics,
       keep_unused=keep_unused,
       inline=inline)
   return arrays_to_spvalues(spenv, tree_unflatten(out_tree, out_flat))
@@ -894,6 +882,8 @@ _bcoo_methods = {
   "__rmatmul__": sparsify(_swap_args(jnp.matmul)),
   "__mul__": sparsify(jnp.multiply),
   "__rmul__": sparsify(_swap_args(jnp.multiply)),
+  "__truediv__": sparsify(jnp.divide),
+  "__rtruediv__": sparsify(_swap_args(jnp.divide)),
   "__add__": sparsify(jnp.add),
   "__radd__": sparsify(_swap_args(jnp.add)),
   "__sub__": sparsify(jnp.subtract),
