@@ -18,24 +18,20 @@ from jax._src.interpreters.pxla import (
   ArrayMappingOrAutoOrUnspecified as ArrayMappingOrAutoOrUnspecified,
   AvalDimSharding as AvalDimSharding,
   Chunked as Chunked,
-  ContextDecorator as ContextDecorator,
   DynamicAxisEnv as DynamicAxisEnv,
   DynamicAxisEnvFrame as DynamicAxisEnvFrame,
-  EMPTY_ENV as EMPTY_ENV,
   EmapInfo as EmapInfo,
   ExecuteReplicated as ExecuteReplicated,
   Index as Index,
   InputsHandler as InputsHandler,
   MapTrace as MapTrace,
   MapTracer as MapTracer,
-  MeshAxisName as MeshAxisName,
   MeshComputation as MeshComputation,
   MeshDimAssignment as MeshDimAssignment,
   MeshExecutable as MeshExecutable,
   NoSharding as NoSharding,
   OpShardingType as OpShardingType,
   OrderedDictType as OrderedDictType,
-  OutputType as OutputType,
   ParallelCallableInfo as ParallelCallableInfo,
   PartitionInfo as PartitionInfo,
   PartitionsOrReplicated as PartitionsOrReplicated,
@@ -44,14 +40,10 @@ from jax._src.interpreters.pxla import (
   PxlaResultHandler as PxlaResultHandler,
   ReplicaInfo as ReplicaInfo,
   Replicated as Replicated,
-  ResourceAxisName as ResourceAxisName,
-  ResourceEnv as ResourceEnv,
   ResultsHandler as ResultsHandler,
   SPMDBatchTrace as SPMDBatchTrace,
   ShardInfo as ShardInfo,
   ShardedAxis as ShardedAxis,
-  ShardedDeviceArray as ShardedDeviceArray,
-  ShardedDeviceArrayBase as ShardedDeviceArrayBase,
   ShardingSpec as ShardingSpec,
   TileManual as TileManual,
   TileVectorize as TileVectorize,
@@ -60,8 +52,6 @@ from jax._src.interpreters.pxla import (
   UnloadedPmapExecutable as UnloadedPmapExecutable,
   Unstacked as Unstacked,
   WeakRefList as WeakRefList,
-  _PositionalSemantics as _PositionalSemantics,
-  _ShardedDeviceArray as _ShardedDeviceArray,
   _UNSPECIFIED as _UNSPECIFIED,
   _create_pmap_sharding_spec as _create_pmap_sharding_spec,
   _get_and_check_device_assignment as _get_and_check_device_assignment,
@@ -73,7 +63,7 @@ from jax._src.interpreters.pxla import (
   array_mapping_to_axis_resources as array_mapping_to_axis_resources,
   array_types as array_types,
   custom_resource_typing_rules as custom_resource_typing_rules,
-  device_put as device_put,
+  device_put as _deprecated_device_put,
   find_partitions as find_partitions,
   find_replicas as find_replicas,
   full_to_shard_p as full_to_shard_p,
@@ -100,20 +90,16 @@ from jax._src.interpreters.pxla import (
   reconcile_num_partitions as reconcile_num_partitions,
   replicate as replicate,
   resource_typecheck as resource_typecheck,
-  sda_array_result_handler as sda_array_result_handler,
   shard_arg_handlers as shard_arg_handlers,
   shard_args as shard_args,
   shard_arg as shard_arg,
   shard_aval as shard_aval,
   shard_aval_handlers as shard_aval_handlers,
   shard_to_full_p as shard_to_full_p,
-  sharding_internal as sharding_internal,
   sharding_spec_sharding_proto as sharding_spec_sharding_proto,
-  show_axes as show_axes,
   spec_to_indices as spec_to_indices,
   spmd_primitive_batchers as spmd_primitive_batchers,
   stage_parallel_callable as stage_parallel_callable,
-  thread_resources as thread_resources,
   tile_aval_nd as tile_aval_nd,
   untile_aval_nd as untile_aval_nd,
   vtile_by_mesh as vtile_by_mesh,
@@ -125,30 +111,27 @@ from jax._src.interpreters.pxla import (
   xla_pmap_impl_lazy as xla_pmap_impl_lazy,
   xla_pmap_p as xla_pmap_p,
 )
+from jax._src.mesh import (
+  MeshAxisName as MeshAxisName,
+  thread_resources as thread_resources,
+)
 
 # Deprecations
 
+from jax._src.mesh import Mesh as _deprecated_Mesh
 from jax._src.interpreters.pxla import (
-  Mesh as _deprecated_Mesh,
   PartitionSpec as _deprecated_PartitionSpec,
+  ShardedDeviceArray as _deprecated_ShardedDeviceArray,
   make_sharded_device_array as _deprecated_make_sharded_device_array,
 )
 
-import typing
-if typing.TYPE_CHECKING:
-  from jax._src.interpreters.pxla import (
-    Mesh as Mesh,
-    PartitionSpec as PartitionSpec,
-    make_sharded_device_array as make_sharded_device_array,
-  )
-del typing
-
 _deprecations = {
     # Added Feb 8, 2023:
     "Mesh": (
         "jax.interpreters.pxla.Mesh is deprecated. Use jax.sharding.Mesh.",
         _deprecated_Mesh,
     ),
+    # Added Feb 8, 2023:
     "PartitionSpec": (
         (
             "jax.interpreters.pxla.PartitionSpec is deprecated. Use "
@@ -156,6 +139,14 @@ _deprecations = {
         ),
         _deprecated_PartitionSpec,
     ),
+    # Added March 15, 2023:
+    "ShardedDeviceArray": (
+        (
+            "jax.interpreters.pxla.ShardedDeviceArray is deprecated. Use "
+            "jax.Array."
+        ),
+        _deprecated_ShardedDeviceArray,
+    ),
     # make_sharded_device_array is deprecated as of March 3, 2023. jax.Array
     # is the default since November 2022.
     "make_sharded_device_array": (
@@ -165,8 +156,26 @@ _deprecations = {
         ),
         _deprecated_make_sharded_device_array,
     ),
+    "device_put": (
+        (
+            "jax.interpreters.pxla.device_put is deprecated. Please use"
+            " jax.device_put."
+        ),
+        _deprecated_device_put,
+    ),
 }
 
-from jax._src.deprecations import deprecation_getattr as _deprecation_getattr
-__getattr__ = _deprecation_getattr(__name__, _deprecations)
-del _deprecation_getattr
+import typing
+if typing.TYPE_CHECKING:
+  from jax._src.mesh import Mesh as Mesh
+  from jax._src.interpreters.pxla import (
+    PartitionSpec as PartitionSpec,
+    ShardedDeviceArray as ShardedDeviceArray,
+    device_put as device_put,
+    make_sharded_device_array as make_sharded_device_array,
+  )
+else:
+  from jax._src.deprecations import deprecation_getattr as _deprecation_getattr
+  __getattr__ = _deprecation_getattr(__name__, _deprecations)
+  del _deprecation_getattr
+del typing
