@@ -18,8 +18,6 @@
 from jax.numpy import fft as fft
 from jax.numpy import linalg as linalg
 
-from jax._src.device_array import DeviceArray as DeviceArray
-
 from jax._src.basearray import Array as ndarray
 
 from jax._src.numpy.lax_numpy import (
@@ -109,6 +107,8 @@ from jax._src.numpy.lax_numpy import (
     float16 as float16,
     float32 as float32,
     float64 as float64,
+    float8_e4m3fn as float8_e4m3fn,
+    float8_e5m2 as float8_e5m2,
     float_ as float_,
     floating as floating,
     fmax as fmax,
@@ -168,7 +168,6 @@ from jax._src.numpy.lax_numpy import (
     logspace as logspace,
     mask_indices as mask_indices,
     matmul as matmul,
-    median as median,
     meshgrid as meshgrid,
     moveaxis as moveaxis,
     msort as msort,
@@ -177,9 +176,6 @@ from jax._src.numpy.lax_numpy import (
     nanargmax as nanargmax,
     nanargmin as nanargmin,
     argpartition as argpartition,
-    nanmedian as nanmedian,
-    nanpercentile as nanpercentile,
-    nanquantile as nanquantile,
     ndim as ndim,
     newaxis as newaxis,
     nonzero as nonzero,
@@ -191,14 +187,12 @@ from jax._src.numpy.lax_numpy import (
     packbits as packbits,
     pad as pad,
     partition as partition,
-    percentile as percentile,
     pi as pi,
     piecewise as piecewise,
     place as place,
     printoptions as printoptions,
     promote_types as promote_types,
     put as put,
-    quantile as quantile,
     ravel as ravel,
     ravel_multi_index as ravel_multi_index,
     repeat as repeat,
@@ -260,11 +254,6 @@ from jax._src.numpy.lax_numpy import (
     zeros_like as zeros_like,
 )
 
-from jax._src.numpy.lax_numpy import (
-  float8_e4m3fn,
-  float8_e5m2,
-)
-
 from jax._src.numpy.index_tricks import (
   c_ as c_,
   index_exp as index_exp,
@@ -300,19 +289,25 @@ from jax._src.numpy.reductions import (
     cumproduct as cumproduct,
     max as max,
     mean as mean,
+    median as median,
     min as min,
     nancumsum as nancumsum,
     nancumprod as nancumprod,
     nanmax as nanmax,
     nanmean as nanmean,
+    nanmedian as nanmedian,
     nanmin as nanmin,
+    nanpercentile as nanpercentile,
     nanprod as nanprod,
+    nanquantile as nanquantile,
     nanstd as nanstd,
     nansum as nansum,
     nanvar as nanvar,
+    percentile as percentile,
     prod as prod,
     product as product,
     ptp as ptp,
+    quantile as quantile,
     sometrue as sometrue,
     std as std,
     sum as sum,
@@ -421,3 +416,28 @@ from jax._src.numpy.ufuncs import (
 )
 
 from jax._src.numpy.vectorize import vectorize as vectorize
+
+# Dynamically register numpy-style methods on JAX arrays.
+from jax._src.numpy.array_methods import register_jax_array_methods
+register_jax_array_methods()
+del register_jax_array_methods
+
+
+# Deprecations
+
+_deprecations = {
+    # Added March 14, 2023:
+    "DeviceArray": (
+        "jax.numpy.DeviceArray is deprecated. Use jax.Array.",
+        ndarray,
+    ),
+}
+
+import typing
+if typing.TYPE_CHECKING:
+  from jax._src.basearray import Array as DeviceArray
+else:
+  from jax._src.deprecations import deprecation_getattr as _deprecation_getattr
+  __getattr__ = _deprecation_getattr(__name__, _deprecations)
+  del _deprecation_getattr
+del typing
